@@ -25,6 +25,7 @@ json Board::StackupLayer::serialize() const
     return j;
 }
 
+
 Board::Board(const UUID &uu, const json &j, Block &iblock, Pool &pool, ViaPadstackProvider &vpp)
     : uuid(uu), block(&iblock), name(j.at("name").get<std::string>()), n_inner_layers(j.value("n_inner_layers", 0))
 {
@@ -96,7 +97,7 @@ Board::Board(const UUID &uu, const json &j, Block &iblock, Pool &pool, ViaPadsta
                 }
             }
             if (valid) {
-                load_and_log(tracks, ObjectType::TRACK, std::forward_as_tuple(u, it.value(), *this),
+                load_and_log(tracks, ObjectType::TRACK, std::forward_as_tuple(u, it.value(), this),
                              Logger::Domain::BOARD);
             }
             else {
@@ -108,7 +109,7 @@ Board::Board(const UUID &uu, const json &j, Block &iblock, Pool &pool, ViaPadsta
         const json &o = j["vias"];
         for (auto it = o.cbegin(); it != o.cend(); ++it) {
             auto u = UUID(it.key());
-            load_and_log(vias, ObjectType::VIA, std::forward_as_tuple(u, it.value(), *this, vpp),
+            load_and_log(vias, ObjectType::VIA, std::forward_as_tuple(u, it.value(), this, &vpp),
                          Logger::Domain::BOARD);
         }
     }
@@ -133,6 +134,14 @@ Board::Board(const UUID &uu, const json &j, Block &iblock, Pool &pool, ViaPadsta
             load_and_log(planes, ObjectType::PLANE, std::forward_as_tuple(u, it.value(), *this), Logger::Domain::BOARD);
         }
     }
+    if (j.count("keepouts")) {
+        const json &o = j["keepouts"];
+        for (auto it = o.cbegin(); it != o.cend(); ++it) {
+            auto u = UUID(it.key());
+            load_and_log(keepouts, ObjectType::KEEPOUT, std::forward_as_tuple(u, it.value(), *this),
+                         Logger::Domain::BOARD);
+        }
+    }
     if (j.count("dimensions")) {
         const json &o = j["dimensions"];
         for (auto it = o.cbegin(); it != o.cend(); ++it) {
@@ -165,6 +174,18 @@ Board::Board(const UUID &uu, const json &j, Block &iblock, Pool &pool, ViaPadsta
             Logger::log_warning("couldn't load fab output settings", Logger::Domain::BOARD, e.what());
         }
     }
+    colors.solder_mask = {0, .5, 0};
+    colors.substrate = {.2, .15, 0};
+    if (j.count("colors")) {
+        try {
+            const auto &o = j.at("colors");
+            colors.solder_mask = color_from_json(o.at("solder_mask"));
+            colors.substrate = color_from_json(o.at("substrate"));
+        }
+        catch (const std::exception &e) {
+            Logger::log_warning("couldn't colors", Logger::Domain::BOARD, e.what());
+        }
+    }
     fab_output_settings.update_for_board(this);
     update_refs(); // fill in smashed texts
 }
@@ -198,6 +219,11 @@ Junction *Board::get_junction(const UUID &uu)
     return &junctions.at(uu);
 }
 
+Polygon *Board::get_polygon(const UUID &uu)
+{
+    return &polygons.at(uu);
+}
+
 const std::map<int, Layer> &Board::get_layers() const
 {
     return layers;
@@ -206,8 +232,9 @@ const std::map<int, Layer> &Board::get_layers() const
 Board::Board(const Board &brd)
     : layers(brd.layers), uuid(brd.uuid), block(brd.block), name(brd.name), polygons(brd.polygons), holes(brd.holes),
       packages(brd.packages), junctions(brd.junctions), tracks(brd.tracks), airwires(brd.airwires), vias(brd.vias),
-      texts(brd.texts), lines(brd.lines), arcs(brd.arcs), planes(brd.planes), dimensions(brd.dimensions),
-      warnings(brd.warnings), rules(brd.rules), fab_output_settings(brd.fab_output_settings), stackup(brd.stackup),
+      texts(brd.texts), lines(brd.lines), arcs(brd.arcs), planes(brd.planes), keepouts(brd.keepouts),
+      dimensions(brd.dimensions), warnings(brd.warnings), rules(brd.rules),
+      fab_output_settings(brd.fab_output_settings), stackup(brd.stackup), colors(brd.colors),
       n_inner_layers(brd.n_inner_layers)
 {
     update_refs();
@@ -232,11 +259,13 @@ void Board::operator=(const Board &brd)
     lines = brd.lines;
     arcs = brd.arcs;
     planes = brd.planes;
+    keepouts = brd.keepouts;
     dimensions = brd.dimensions;
     warnings = brd.warnings;
     rules = brd.rules;
     fab_output_settings = brd.fab_output_settings;
     stackup = brd.stackup;
+    colors = brd.colors;
     update_refs();
 }
 
@@ -257,9 +286,6 @@ void Board::update_refs()
     for (auto &it : airwires) {
         it.second.update_refs(*this);
     }
-    for (auto &it : net_segments) {
-        it.second.update(block->nets);
-    }
     for (auto &it : vias) {
         it.second.junction.update(junctions);
         it.second.net_set.update(block->nets);
@@ -286,6 +312,10 @@ void Board::update_refs()
         it.second.polygon.update(polygons);
         it.second.polygon->usage = &it.second;
     }
+    for (auto &it : keepouts) {
+        it.second.polygon.update(polygons);
+        it.second.polygon->usage = &it.second;
+    }
 }
 
 unsigned int Board::get_n_inner_layers() const
@@ -297,27 +327,31 @@ void Board::set_n_inner_layers(unsigned int n)
 {
     n_inner_layers = n;
     layers.clear();
-    layers = {{100, {100, "Outline", {.6, .6, 0}}},
-              {60, {60, "Top Courtyard", {.5, .5, .5}}},
-              {50, {50, "Top Assembly", {.5, .5, .5}}},
-              {40, {40, "Top Package", {.5, .5, .5}}},
-              {30, {30, "Top Paste", {.8, .8, .8}}},
-              {20, {20, "Top Silkscreen", {.9, .9, .9}}},
-              {10, {10, "Top Mask", {1, .5, .5}}},
-              {0, {0, "Top Copper", {1, 0, 0}, false, true}},
-              {-100, {-100, "Bottom Copper", {0, .5, 0}, true, true}},
-              {-110, {-110, "Bottom Mask", {.25, .5, .25}, true}},
-              {-120, {-120, "Bottom Silkscreen", {.9, .9, .9}, true}},
-              {-130, {-130, "Bottom Paste", {.8, .8, .8}}},
-              {-140, {-140, "Bottom Package", {.5, .5, .5}}},
-              {-150, {-150, "Bottom Assembly", {.5, .5, .5}, true}},
-              {-160, {-160, "Bottom Courtyard", {.5, .5, .5}}}};
-    stackup.emplace(0, 0);
+    layers = {{200, {200, "Top Notes"}},
+              {100, {100, "Outline"}},
+              {60, {60, "Top Courtyard"}},
+              {50, {50, "Top Assembly"}},
+              {40, {40, "Top Package"}},
+              {30, {30, "Top Paste"}},
+              {20, {20, "Top Silkscreen"}},
+              {10, {10, "Top Mask"}},
+              {0, {0, "Top Copper", false, true}},
+              {-100, {-100, "Bottom Copper", true, true}},
+              {-110, {-110, "Bottom Mask", true}},
+              {-120, {-120, "Bottom Silkscreen", true}},
+              {-130, {-130, "Bottom Paste"}},
+              {-140, {-140, "Bottom Package"}},
+              {-150, {-150, "Bottom Assembly", true}},
+              {-160, {-160, "Bottom Courtyard"}},
+              {-200, {-200, "Bottom Notes", true}}};
+    if (stackup.emplace(0, 0).second) { // if created
+        stackup.at(0).substrate_thickness = 1.6_mm;
+    }
     stackup.emplace(-100, -100);
     stackup.at(-100).substrate_thickness = 0;
     for (unsigned int i = 0; i < n_inner_layers; i++) {
         auto j = i + 1;
-        layers.emplace(std::make_pair(-j, Layer(-j, "Inner " + std::to_string(j), {1, 1, 0}, false, true)));
+        layers.emplace(std::make_pair(-j, Layer(-j, "Inner " + std::to_string(j), false, true)));
         stackup.emplace(-j, -j);
     }
     map_erase_if(stackup, [this](const auto &x) { return layers.count(x.first) == 0; });
@@ -330,102 +364,68 @@ static void flip_package_layer(int &layer)
     layer = -layer - 100;
 }
 
-bool Board::propagate_net_segments()
+void Board::propagate_nets()
 {
-    net_segments.clear();
-    net_segments.emplace(UUID(), nullptr);
+    // reset
+    for (auto &it : tracks) {
+        it.second.update_refs(*this);
+        it.second.net = nullptr;
+    }
+    for (auto &it : junctions) {
+        it.second.net = nullptr;
+    }
 
-    bool run = true;
-    while (run) {
-        run = false;
-        for (auto &it_pkg : packages) { // find one pad with net, but without net segment and
-                                        // assign it to a new net segment
-            for (auto &it_pad : it_pkg.second.package.pads) {
-                if (!it_pad.second.net_segment && it_pad.second.net) {
-                    it_pad.second.net_segment = UUID::random();
-                    net_segments.emplace(it_pad.second.net_segment, it_pad.second.net);
-                    run = true;
-                    break;
-                }
-            }
-            if (run)
-                break;
+    for (auto &it_via : vias) {
+        auto ju = it_via.second.junction;
+        if (it_via.second.net_set) {
+            ju->net = it_via.second.net_set;
         }
-        if (run == false) { // no pad needing assignment found, look at vias
-            for (auto &it_via : vias) {
-                auto ju = it_via.second.junction;
-                if (!ju->net_segment && it_via.second.net_set) {
-                    ju->net_segment = UUID::random();
-                    net_segments.emplace(ju->net_segment, it_via.second.net_set);
-                    run = true;
-                    break;
-                }
-            }
-        }
-        if (run == false) // done with pads and vias
-            break;
+    }
 
-        unsigned int n_assigned = 1;
-        while (n_assigned) {
-            n_assigned = 0;
-            for (auto &it : tracks) {
-                if (it.second.net_segment) { // track with net seg
-                    for (auto &it_ft : {it.second.from, it.second.to}) {
-                        if (it_ft.is_junc() && !it_ft.junc->net_segment) { // propgate to junction
-                            it_ft.junc->net_segment = it.second.net_segment;
-                            n_assigned++;
+    bool assigned = true;
+    std::set<const Track *> tracks_erase;
+    while (assigned) {
+        assigned = false;
+        for (auto &it : tracks) {
+            if (it.second.net) {
+                for (const auto &it_ft : {it.second.from, it.second.to}) {
+                    if (it_ft.is_pad()) {
+                        if (it_ft.pad->net != it.second.net) { // connected to wrong pad, delete
+                            tracks_erase.insert(&it.second);
                         }
-                        else if (it_ft.is_pad() && !it_ft.pad->net_segment) { // propagate to
-                                                                              // pad
-                            it_ft.pad->net_segment = it.second.net_segment;
-                            n_assigned++;
+                    }
+                    else if (it_ft.is_junc()) {
+                        if (it_ft.junc->net
+                            && (it_ft.junc->net != it.second.net)) { // connected to wrong junction, delete
+                            tracks_erase.insert(&it.second);
+                        }
+                        else if (it_ft.junc->net == nullptr) {
+                            it_ft.junc->net = it.second.net;
+                            assigned = true;
                         }
                     }
                 }
-                else { // track without net seg, so propagate net segment to
-                       // track
-                    for (auto &it_ft : {it.second.from, it.second.to}) {
-                        if (it_ft.is_junc() && it_ft.junc->net_segment) { // propagate from
-                                                                          // junction
-                            it.second.net_segment = it_ft.junc->net_segment;
-                            n_assigned++;
+            }
+            else { // no net, take net from pad/junc
+                for (const auto &it_ft : {it.second.from, it.second.to}) {
+                    if (it_ft.is_pad()) {
+                        if (it_ft.pad->net) {
+                            it.second.net = it_ft.pad->net;
+                            assigned = true;
                         }
-                        else if (it_ft.is_pad() && it_ft.pad->net_segment) {
-                            it.second.net_segment = it_ft.pad->net_segment; // propgate from pad
-                            // it_ft.pin->connected_net_lines.emplace(it.first,
-                            // &it.second);
-                            n_assigned++;
+                    }
+                    else if (it_ft.is_junc()) {
+                        if (it_ft.junc->net) {
+                            it.second.net = it_ft.junc->net;
+                            assigned = true;
                         }
                     }
                 }
             }
         }
     }
-    for (auto &it : junctions) { // assign net to junctions
-        it.second.net = net_segments.at(it.second.net_segment);
-    }
-
-    bool done = true;
-    for (auto it = tracks.begin(); it != tracks.end();) { // find tracks that connect different nets and
-                                                          // delete these
-        it->second.net = net_segments.at(it->second.net_segment);
-        bool erased = false;
-        for (auto &it_ft : {it->second.from, it->second.to}) {
-            if (it_ft.is_pad() && it_ft.pad->net) {
-                if (it->second.net != it_ft.pad->net) {
-                    done = false;
-                    tracks.erase(it++);
-                    erased = true;
-                    break;
-                }
-            }
-        }
-        if (!erased)
-            it++;
-    }
-    // if a track has been deleted, we'll have to re-run the net segment
-    // assignment again
-    return done;
+    if (tracks_erase.size())
+        map_erase_if(tracks, [&tracks_erase](const auto &it) { return tracks_erase.count(&it.second); });
 }
 
 void Board::vacuum_junctions()
@@ -534,24 +534,13 @@ void Board::expand(bool careful)
     for (auto &it : planes) {
         it.second.polygon->usage = &it.second;
     }
+    for (auto &it : keepouts) {
+        it.second.polygon->usage = &it.second;
+    }
 
-    do {
-        // clear net segments and nets assigned to tracks and junctions
-        for (auto &it : packages) {
-            for (auto &it_pad : it.second.package.pads) {
-                it_pad.second.net_segment = UUID();
-            }
-        }
-        for (auto &it : tracks) {
-            it.second.update_refs(*this);
-            it.second.net = nullptr;
-            it.second.net_segment = UUID();
-        }
-        for (auto &it : junctions) {
-            it.second.net = nullptr;
-            it.second.net_segment = UUID();
-        }
-    } while (propagate_net_segments() == false); // run as long as propagate net_segments deletes tracks
+    if (expand_flags & EXPAND_PROPAGATE_NETS) {
+        propagate_nets();
+    }
 
     for (auto &it : vias) {
         if (it.second.junction->net == nullptr && !it.second.net_set) {
@@ -562,7 +551,11 @@ void Board::expand(bool careful)
         }
     }
 
-    update_airwires();
+    if (expand_flags & EXPAND_AIRWIRES)
+        update_airwires();
+
+    expand_flags = EXPAND_ALL;
+    packages_expand.clear();
 }
 
 void Board::expand_packages()
@@ -576,75 +569,80 @@ void Board::expand_packages()
 
     };
 
+    bool expanded_package = false;
     for (auto &it : packages) {
         it.second.pool_package = it.second.component->part->package;
-        it.second.model = it.second.component->part->model;
-        if (it.second.alternate_package) {
-            std::set<std::string> pads_from_primary, pads_from_alt;
-            for (const auto &it_pad : it.second.pool_package->pads) {
-                pads_from_primary.insert(it_pad.second.name);
-            }
-            bool alt_valid = true;
-            for (const auto &it_pad : it.second.alternate_package->pads) {
-                if (!pads_from_alt.insert(it_pad.second.name).second) { // duplicate pad name
-                    alt_valid = false;
+        it.second.model = it.second.component->part->get_model();
+        if ((expand_flags & EXPAND_PACKAGES) && (packages_expand.size() == 0 || packages_expand.count(it.first))) {
+            expanded_package = true;
+            if (it.second.alternate_package) {
+                std::set<std::string> pads_from_primary, pads_from_alt;
+                for (const auto &it_pad : it.second.pool_package->pads) {
+                    pads_from_primary.insert(it_pad.second.name);
+                }
+                bool alt_valid = true;
+                for (const auto &it_pad : it.second.alternate_package->pads) {
+                    if (!pads_from_alt.insert(it_pad.second.name).second) { // duplicate pad name
+                        alt_valid = false;
+                    }
+                }
+                if (!alt_valid || pads_from_alt != pads_from_primary) { // alt pkg isn't pad-equal
+                    it.second.package = *it.second.pool_package;
+                    warnings.emplace_back(it.second.placement.shift, "Incompatible alt pkg");
+                }
+                else {
+                    it.second.package = *it.second.alternate_package;
+                    it.second.package.pads.clear(); // need to adjust pad uuids to primary package
+                    std::map<std::string, UUID> pad_uuids;
+                    for (const auto &it_pad : it.second.pool_package->pads) {
+                        assert(pad_uuids.emplace(it_pad.second.name, it_pad.first).second); // no duplicates
+                    }
+                    for (const auto &it_pad : it.second.alternate_package->pads) {
+                        auto uu = pad_uuids.at(it_pad.second.name);
+                        auto &pad = it.second.package.pads.emplace(uu, it_pad.second).first->second;
+                        pad.uuid = uu;
+                    }
                 }
-            }
-            if (!alt_valid || pads_from_alt != pads_from_primary) { // alt pkg isn't pad-equal
-                it.second.package = *it.second.pool_package;
-                warnings.emplace_back(it.second.placement.shift, "Incompatible alt pkg");
             }
             else {
-                it.second.package = *it.second.alternate_package;
-                it.second.package.pads.clear(); // need to adjust pad uuids to primary package
-                std::map<std::string, UUID> pad_uuids;
-                for (const auto &it_pad : it.second.pool_package->pads) {
-                    assert(pad_uuids.emplace(it_pad.second.name, it_pad.first).second); // no duplicates
+                it.second.package = *it.second.pool_package;
+            }
+
+            it.second.placement.mirror = it.second.flip;
+            for (auto &it2 : it.second.package.pads) {
+                it2.second.padstack.expand_inner(n_inner_layers);
+            }
+
+            if (it.second.flip) {
+                for (auto &it2 : it.second.package.lines) {
+                    flip_package_layer(it2.second.layer);
                 }
-                for (const auto &it_pad : it.second.alternate_package->pads) {
-                    auto uu = pad_uuids.at(it_pad.second.name);
-                    auto &pad = it.second.package.pads.emplace(uu, it_pad.second).first->second;
-                    pad.uuid = uu;
+                for (auto &it2 : it.second.package.arcs) {
+                    flip_package_layer(it2.second.layer);
+                }
+                for (auto &it2 : it.second.package.texts) {
+                    flip_package_layer(it2.second.layer);
+                }
+                for (auto &it2 : it.second.package.polygons) {
+                    flip_package_layer(it2.second.layer);
+                }
+                for (auto &it2 : it.second.package.pads) {
+                    if (it2.second.padstack.type == Padstack::Type::TOP) {
+                        it2.second.padstack.type = Padstack::Type::BOTTOM;
+                    }
+                    else if (it2.second.padstack.type == Padstack::Type::BOTTOM) {
+                        it2.second.padstack.type = Padstack::Type::TOP;
+                    }
+                    for (auto &it3 : it2.second.padstack.polygons) {
+                        flip_package_layer(it3.second.layer);
+                    }
+                    for (auto &it3 : it2.second.padstack.shapes) {
+                        flip_package_layer(it3.second.layer);
+                    }
                 }
             }
         }
-        else {
-            it.second.package = *it.second.pool_package;
-        }
         auto r = it.second.package.apply_parameter_set(pset);
-        it.second.placement.mirror = it.second.flip;
-        for (auto &it2 : it.second.package.pads) {
-            it2.second.padstack.expand_inner(n_inner_layers);
-        }
-
-        if (it.second.flip) {
-            for (auto &it2 : it.second.package.lines) {
-                flip_package_layer(it2.second.layer);
-            }
-            for (auto &it2 : it.second.package.arcs) {
-                flip_package_layer(it2.second.layer);
-            }
-            for (auto &it2 : it.second.package.texts) {
-                flip_package_layer(it2.second.layer);
-            }
-            for (auto &it2 : it.second.package.polygons) {
-                flip_package_layer(it2.second.layer);
-            }
-            for (auto &it2 : it.second.package.pads) {
-                if (it2.second.padstack.type == Padstack::Type::TOP) {
-                    it2.second.padstack.type = Padstack::Type::BOTTOM;
-                }
-                else if (it2.second.padstack.type == Padstack::Type::BOTTOM) {
-                    it2.second.padstack.type = Padstack::Type::TOP;
-                }
-                for (auto &it3 : it2.second.padstack.polygons) {
-                    flip_package_layer(it3.second.layer);
-                }
-                for (auto &it3 : it2.second.padstack.shapes) {
-                    flip_package_layer(it3.second.layer);
-                }
-            }
-        }
 
         it.second.texts.erase(std::remove_if(it.second.texts.begin(), it.second.texts.end(),
                                              [this](const auto &a) { return texts.count(a.uuid) == 0; }),
@@ -657,6 +655,10 @@ void Board::expand_packages()
             it_text->text_override = it.second.replace_text(it_text->text, &it_text->overridden);
         }
     }
+
+    if (expanded_package)
+        update_refs();
+
     // assign nets to pads based on netlist
     for (auto &it : packages) {
         for (auto &it_pad : it.second.package.pads) {
@@ -745,6 +747,37 @@ void Board::delete_dependants()
 {
     map_erase_if(vias, [this](auto &it) { return junctions.count(it.second.junction.uuid) == 0; });
     map_erase_if(planes, [this](auto &it) { return polygons.count(it.second.polygon.uuid) == 0; });
+    map_erase_if(keepouts, [this](auto &it) { return polygons.count(it.second.polygon.uuid) == 0; });
+}
+
+std::vector<KeepoutContour> Board::get_keepout_contours() const
+{
+    std::vector<KeepoutContour> r;
+    for (const auto &it : keepouts) {
+        r.emplace_back();
+        ClipperLib::Path &contour = r.back().contour;
+        r.back().keepout = &it.second;
+        auto poly2 = it.second.polygon->remove_arcs();
+        contour.reserve(poly2.vertices.size());
+        for (const auto &itv : poly2.vertices) {
+            contour.push_back({itv.position.x, itv.position.y});
+        }
+    }
+    for (const auto &it_pkg : packages) {
+        for (const auto &it : it_pkg.second.package.keepouts) {
+            r.emplace_back();
+            ClipperLib::Path &contour = r.back().contour;
+            r.back().keepout = &it.second;
+            r.back().pkg = &it_pkg.second;
+            auto poly2 = it.second.polygon->remove_arcs();
+            contour.reserve(poly2.vertices.size());
+            for (const auto &itv : poly2.vertices) {
+                auto p = it_pkg.second.placement.transform(itv.position);
+                contour.push_back({p.x, p.y});
+            }
+        }
+    }
+    return r;
 }
 
 json Board::serialize() const
@@ -757,7 +790,12 @@ json Board::serialize() const
     j["n_inner_layers"] = n_inner_layers;
     j["rules"] = rules.serialize();
     j["fab_output_settings"] = fab_output_settings.serialize();
-
+    {
+        json o;
+        o["solder_mask"] = color_to_json(colors.solder_mask);
+        o["substrate"] = color_to_json(colors.substrate);
+        j["colors"] = o;
+    }
     j["polygons"] = json::object();
     for (const auto &it : polygons) {
         j["polygons"][(std::string)it.first] = it.second.serialize();
@@ -806,6 +844,10 @@ json Board::serialize() const
     for (const auto &it : dimensions) {
         j["dimensions"][(std::string)it.first] = it.second.serialize();
     }
+    j["keepouts"] = json::object();
+    for (const auto &it : keepouts) {
+        j["keepouts"][(std::string)it.first] = it.second.serialize();
+    }
     return j;
 }
 } // namespace horizon
