@@ -1,9 +1,12 @@
 #pragma once
+#include <string>
+#include <vector>
 
 namespace horizon {
 class BoardLayers {
 public:
     enum Layer {
+        TOP_NOTES = 200,
         L_OUTLINE = 100,
         TOP_COURTYARD = 60,
         TOP_ASSEMBLY = 50,
@@ -23,11 +26,15 @@ public:
         BOTTOM_PACKAGE = -140,
         BOTTOM_ASSEMBLY = -150,
         BOTTOM_COURTYARD = -160,
+        BOTTOM_NOTES = -200
     };
 
     static bool is_copper(int l)
     {
         return l <= TOP_COPPER && l >= BOTTOM_COPPER;
     }
+
+    static std::string get_layer_name(int l);
+    static const std::vector<int> &get_layers();
 };
 } // namespace horizon
