@@ -22,7 +22,7 @@ void Canvas::remove_annotation(CanvasAnnotation *a)
 
 CanvasAnnotation::CanvasAnnotation(Canvas *c, int l) : ca(c), layer(l)
 {
-    LayerDisplay ld(false, LayerDisplay::Mode::FILL_ONLY, {1, 1, 0});
+    LayerDisplay ld(false, LayerDisplay::Mode::FILL_ONLY);
     ca->set_layer_display(layer, ld);
 }
 
@@ -45,5 +45,6 @@ void CanvasAnnotation::clear()
 void CanvasAnnotation::draw_line(const Coordi &from, const Coordi &to, ColorP color, uint64_t width)
 {
     ca->draw_line(from, to, color, layer, false, width);
+    ca->request_push();
 }
 } // namespace horizon
