@@ -15,11 +15,11 @@
 #include "pool/symbol.hpp"
 #include "schematic/bus_label.hpp"
 #include "schematic/bus_ripper.hpp"
-#include "schematic/frame.hpp"
 #include "schematic/line_net.hpp"
 #include "schematic/net_label.hpp"
 #include "schematic/power_symbol.hpp"
 #include "schematic/sheet.hpp"
+#include "frame/frame.hpp"
 #include "selectables.hpp"
 #include "selection_filter.hpp"
 #include "target.hpp"
@@ -32,15 +32,9 @@
 
 namespace horizon {
 
-Color Canvas::get_layer_color(int layer)
-{
-    Color c = layer_provider->get_layers().at(layer).color;
-    return c;
-}
-
 void Canvas::render(const Junction &junc, bool interactive, ObjectType mode)
 {
-    ColorP c = ColorP::YELLOW;
+    ColorP c = ColorP::JUNCTION;
     if (junc.net) {
         if (junc.net->diffpair)
             c = ColorP::DIFFPAIR;
@@ -69,6 +63,9 @@ void Canvas::render(const Junction &junc, bool interactive, ObjectType mode)
         else if (junc.connection_count >= 3 && mode == ObjectType::SCHEMATIC) {
             draw_line(junc.position, junc.position + Coordi(0, 1000), c, 0, true, 0.75_mm);
         }
+        else if (junc.has_via && mode == ObjectType::SCHEMATIC) {
+            // nop
+        }
         else {
             draw_cross(junc.position, 0.25_mm, c);
         }
@@ -91,17 +88,28 @@ void Canvas::render(const PowerSymbol &sym)
 
     auto style = sym.net->power_symbol_style;
     switch (style) {
-    case Net::PowerSymbolStyle::GND: {
+    case Net::PowerSymbolStyle::GND:
+    case Net::PowerSymbolStyle::EARTH: {
+        transform_save();
+        transform.set_angle(orientation_to_angle(sym.orientation) - 49152);
         draw_line({0, 0}, {0, -1.25_mm}, c, 0);
-        draw_line({-1.25_mm, -1.25_mm}, {1.25_mm, -1.25_mm}, c, 0);
-        draw_line({-1.25_mm, -1.25_mm}, {0, -2.5_mm}, c, 0);
-        draw_line({1.25_mm, -1.25_mm}, {0, -2.5_mm}, c, 0);
+        if (style == Net::PowerSymbolStyle::GND) {
+            draw_line({-1.25_mm, -1.25_mm}, {1.25_mm, -1.25_mm}, c, 0);
+            draw_line({-1.25_mm, -1.25_mm}, {0, -2.5_mm}, c, 0);
+            draw_line({1.25_mm, -1.25_mm}, {0, -2.5_mm}, c, 0);
+        }
+        else {
+            for (float i = 0.0_mm; i < 1.25_mm; i += 1.50_mm / 3) {
+                draw_line({-1.25_mm + i, -1.25_mm - i}, {1.25_mm - i, -1.25_mm - i}, c, 0);
+            }
+        }
+        selectables.append(sym.uuid, ObjectType::POWER_SYMBOL, {0, 0}, {-1.25_mm, -2.5_mm}, {1.25_mm, 0_mm});
+        transform_restore();
 
         if (sym.orientation != Orientation::DOWN) {
             draw_error({0, 0}, 2e5, "Unsupported orientation", true);
         }
 
-        selectables.append(sym.uuid, ObjectType::POWER_SYMBOL, {0, 0}, {-1.25_mm, -2.5_mm}, {1.25_mm, 0_mm});
         transform.reset();
 
         int text_angle = 0;
@@ -117,28 +125,47 @@ void Canvas::render(const PowerSymbol &sym)
 
     case Net::PowerSymbolStyle::DOT:
     case Net::PowerSymbolStyle::ANTENNA: {
-        float dir = sym.orientation == Orientation::DOWN ? -1 : 1;
-        if (sym.orientation == Orientation::LEFT || sym.orientation == Orientation::RIGHT) {
-            draw_error({0, 0}, 2e5, "Unsupported orientation", true);
-        }
-
+        transform_save();
+        transform.set_angle(orientation_to_angle(sym.orientation) - 16384);
         if (style == Net::PowerSymbolStyle::ANTENNA) {
-            draw_line({0, 0}, {0, dir * 2.5_mm}, c, 0);
-            draw_line({-1_mm, dir * 1_mm}, {0, dir * 2.5_mm}, c, 0);
-            draw_line({1_mm, dir * 1_mm}, {0, dir * 2.5_mm}, c, 0);
-            selectables.append(sym.uuid, ObjectType::POWER_SYMBOL, {0, 0}, {-1_mm, dir * 2.5_mm}, {1_mm, 0_mm});
+            draw_line({0, 0}, {0, 2.5_mm}, c, 0);
+            draw_line({-1_mm, 1_mm}, {0, 2.5_mm}, c, 0);
+            draw_line({1_mm, 1_mm}, {0, 2.5_mm}, c, 0);
+            selectables.append(sym.uuid, ObjectType::POWER_SYMBOL, {0, 0}, {-1_mm, 2.5_mm}, {1_mm, 0_mm});
         }
         else {
-            draw_line({0, 0}, {0, dir * 1_mm}, c, 0);
-            draw_arc({0, dir * 1.75_mm}, 0.75_mm, 0, 2 * M_PI, ColorP::FROM_LAYER, 0, true, 0);
-            selectables.append(sym.uuid, ObjectType::POWER_SYMBOL, {0, 0}, {-.75_mm, dir * 2.5_mm}, {.75_mm, 0_mm});
+            draw_line({0, 0}, {0, 1_mm}, c, 0);
+            draw_arc({0, 1.75_mm}, 0.75_mm, 0, 2 * M_PI, ColorP::FROM_LAYER, 0, true, 0);
+            selectables.append(sym.uuid, ObjectType::POWER_SYMBOL, {0, 0}, {-.75_mm, 2.5_mm}, {.75_mm, 0_mm});
         }
+        transform_restore();
 
         transform.reset();
 
+        Coordi text_offset;
+        bool mirror = false;
+        const int64_t center_offset = 1.25_mm;
+        const int64_t radius = 1.875_mm;
+        switch (sym.orientation) {
+        case Orientation::UP:
+            text_offset = {center_offset, radius};
+            mirror = not sym.mirror;
+            break;
+        case Orientation::RIGHT:
+            text_offset = {center_offset + radius, 0.0_mm};
+            break;
+        case Orientation::DOWN:
+            text_offset = {center_offset, -radius};
+            mirror = sym.mirror;
+            break;
+        case Orientation::LEFT:
+            text_offset = {center_offset + radius, 0.0_mm};
+            mirror = true;
+            break;
+        }
+
         int text_angle = 0;
-        Coordi text_offset(1.25_mm, dir * 1.875_mm);
-        if (sym.mirror ^ (dir > 0)) {
+        if (mirror) {
             text_offset.x *= -1;
             text_angle = 32768;
         }
@@ -153,21 +180,6 @@ void Canvas::render(const PowerSymbol &sym)
     object_refs_current.pop_back();
 }
 
-static auto get_line_bb(const Coordf &from, const Coordf &to, float width)
-{
-    auto center = (from + to) / 2;
-    auto v = to - from;
-    float length = sqrt(v.mag_sq());
-    auto phi = atan2f(v.y, v.x);
-    auto bb = std::make_pair(Coordf(-length / 2 - width / 2, -width / 2), Coordf(length / 2 + width / 2, width / 2));
-    Placement tr;
-    tr.set_angle(phi / (2 * M_PI) * 65536);
-    auto bbt = tr.transform_bb(bb);
-    bbt.first += center;
-    bbt.second += center;
-    return bbt;
-}
-
 ObjectRef Canvas::add_line(const std::deque<Coordi> &pts, int64_t width, ColorP color, int layer)
 {
     auto uu = UUID::random();
@@ -199,10 +211,8 @@ void Canvas::render(const Line &line, bool interactive)
     draw_line(line.from->position, line.to->position, ColorP::FROM_LAYER, line.layer, true, line.width);
     triangle_type_current = Triangle::Type::NONE;
     if (interactive) {
-        auto center = (line.from->position + line.to->position) / 2;
-        auto bb = get_line_bb(line.from->position, line.to->position, line.width);
-
-        selectables.append(line.uuid, ObjectType::LINE, center, bb.first, bb.second, 0, line.layer);
+        selectables.append_line(line.uuid, ObjectType::LINE, line.from->position, line.to->position, line.width, 0,
+                                line.layer);
     }
 }
 
@@ -226,9 +236,7 @@ void Canvas::render(const LineNet &line)
     object_refs_current.emplace_back(ObjectType::LINE_NET, line.uuid);
     draw_line(line.from.get_position(), line.to.get_position(), c, 0, true, width);
     object_refs_current.pop_back();
-    selectables.append(line.uuid, ObjectType::LINE_NET, (line.from.get_position() + line.to.get_position()) / 2,
-                       Coordf::min(line.from.get_position(), line.to.get_position()),
-                       Coordf::max(line.from.get_position(), line.to.get_position()));
+    selectables.append_line(line.uuid, ObjectType::LINE_NET, line.from.get_position(), line.to.get_position(), width);
 }
 
 void Canvas::render(const Track &track)
@@ -259,17 +267,12 @@ void Canvas::render(const Track &track)
     draw_line(track.from.get_position(), track.to.get_position(), c, layer, true, width);
     if (track.locked && !track.is_air) {
         auto ol = get_overlay_layer(layer);
-        draw_lock(center, 0.7 * track.width, ColorP::WHITE, ol, true);
+        draw_lock(center, 0.7 * track.width, ColorP::TEXT_OVERLAY, ol, true);
     }
     object_refs_current.pop_back();
     if (!track.is_air) {
-
-        float box_height = track.width;
-        Coordf delta = track.to.get_position() - track.from.get_position();
-        float box_width = track.width + sqrt(delta.mag_sq());
-        float angle = atan2(delta.y, delta.x);
-        selectables.append_angled(track.uuid, ObjectType::TRACK, center, center, Coordf(box_width, box_height), angle,
-                                  0, track.layer);
+        selectables.append_line(track.uuid, ObjectType::TRACK, track.from.get_position(), track.to.get_position(),
+                                track.width, 0, track.layer);
     }
 }
 
@@ -280,15 +283,18 @@ void Canvas::render(const SymbolPin &pin, bool interactive)
 
     Coordi p_name = p0;
     Coordi p_pad = p0;
+    Coordi p_nc = p0;
 
     Orientation pin_orientation = pin.get_orientation_for_placement(transform);
 
     Orientation name_orientation = Orientation::LEFT;
     Orientation pad_orientation = Orientation::LEFT;
+    Orientation nc_orientation = Orientation::LEFT;
     int64_t text_shift = 0.5_mm;
     int64_t text_shift_name = text_shift;
     int64_t schmitt_shift = 1.125_mm;
     int64_t driver_shift = .75_mm;
+    int64_t nc_shift = 0.25_mm;
     int64_t length = pin.length;
     auto dot_size = .75_mm;
     if (pin.decoration.dot) {
@@ -315,6 +321,8 @@ void Canvas::render(const SymbolPin &pin, bool interactive)
         p_pad.x += pin.length - text_shift;
         p_pad.y += text_shift;
         v_deco.x = 1;
+        p_nc.x -= nc_shift;
+        nc_orientation = Orientation::LEFT;
         name_orientation = Orientation::RIGHT;
         pad_orientation = Orientation::LEFT;
         break;
@@ -325,6 +333,8 @@ void Canvas::render(const SymbolPin &pin, bool interactive)
         p_pad.x -= pin.length - text_shift;
         p_pad.y += text_shift;
         v_deco.x = -1;
+        p_nc.x += nc_shift;
+        nc_orientation = Orientation::RIGHT;
         name_orientation = Orientation::LEFT;
         pad_orientation = Orientation::RIGHT;
         break;
@@ -335,6 +345,8 @@ void Canvas::render(const SymbolPin &pin, bool interactive)
         p_pad.y -= pin.length - text_shift;
         p_pad.x -= text_shift;
         v_deco.y = -1;
+        p_nc.y += nc_shift;
+        nc_orientation = Orientation::UP;
         name_orientation = Orientation::DOWN;
         pad_orientation = Orientation::UP;
         break;
@@ -345,6 +357,8 @@ void Canvas::render(const SymbolPin &pin, bool interactive)
         p_pad.y += pin.length - text_shift;
         p_pad.x -= text_shift;
         v_deco.y = 1;
+        p_nc.y -= nc_shift;
+        nc_orientation = Orientation::DOWN;
         name_orientation = Orientation::UP;
         pad_orientation = Orientation::DOWN;
         break;
@@ -358,8 +372,22 @@ void Canvas::render(const SymbolPin &pin, bool interactive)
         c_pad = ColorP::PIN_HIDDEN;
     }
     if (interactive || pin.name_visible) {
-        draw_text0(p_name, 1.5_mm, pin.name, orientation_to_angle(name_orientation), false, TextOrigin::CENTER, c_name,
-                   0);
+        bool draw_in_line = pin.name_orientation == SymbolPin::NameOrientation::IN_LINE
+                            || (pin.name_orientation == SymbolPin::NameOrientation::HORIZONTAL
+                                && (pin_orientation == Orientation::LEFT || pin_orientation == Orientation::RIGHT));
+
+        if (draw_in_line) {
+            draw_text0(p_name, 1.5_mm, pin.name, orientation_to_angle(name_orientation), false, TextOrigin::CENTER,
+                       c_name, 0);
+        }
+        else { // draw perp
+            int64_t yshift = 0;
+            Placement tr;
+            tr.set_angle(orientation_to_angle(pin_orientation));
+            auto shift = tr.transform(Coordi(-1_mm, 0));
+            draw_text0(p_name + shift, 1.5_mm, pin.name, orientation_to_angle(name_orientation) + 16384, false,
+                       TextOrigin::CENTER, c_name, 0, 0, true, TextData::Font::SIMPLEX, true);
+        }
     }
     std::pair<Coordf, Coordf> pad_extents;
     if (interactive || pin.pad_visible) {
@@ -493,6 +521,11 @@ void Canvas::render(const SymbolPin &pin, bool interactive)
         draw_box(p0, 0.25_mm, ColorP::FROM_LAYER, 0, false);
         break;
 
+    case SymbolPin::ConnectorStyle::NC:
+        draw_cross(p0, 0.25_mm, ColorP::FROM_LAYER, 0, false);
+        draw_text0(p_nc, 1.5_mm, "NC", orientation_to_angle(nc_orientation), false, TextOrigin::CENTER, ColorP::PIN, 0);
+        break;
+
     case SymbolPin::ConnectorStyle::NONE:
         break;
     }
@@ -519,10 +552,10 @@ void Canvas::render(const Arc &arc, bool interactive)
     float radius1 = sqrt(sq(c.x - b.x) + sq(c.y - b.y));
     float a0 = atan2f(a.y - c.y, a.x - c.x);
     float a1 = atan2f(b.y - c.y, b.x - c.x);
-    draw_arc2(c, radius0, a0, radius1, a1, ColorP::FROM_LAYER, arc.layer, true, arc.width);
+    auto bb = draw_arc2(c, radius0, a0, radius1, a1, ColorP::FROM_LAYER, arc.layer, true, arc.width);
     Coordf t(radius0, radius0);
     if (interactive)
-        selectables.append(arc.uuid, ObjectType::ARC, c, c - t, c + t, 0, arc.layer);
+        selectables.append(arc.uuid, ObjectType::ARC, c, bb.first, bb.second, 0, arc.layer);
 }
 
 void Canvas::render(const SchematicSymbol &sym)
@@ -550,15 +583,13 @@ void Canvas::render(const Text &text, bool interactive, bool reorient)
         angle = 32768 - angle;
     }
 
-    img_text_layer(text.layer);
     img_patch_type(PatchType::TEXT);
     triangle_type_current = Triangle::Type::TEXT;
     auto extents = draw_text0(transform.shift, text.size, text.overridden ? text.text_override : text.text, angle, rev,
-                              text.origin, ColorP::FROM_LAYER, text.layer, text.width);
+                              text.origin, ColorP::FROM_LAYER, text.layer, text.width, true, text.font);
     triangle_type_current = Triangle::Type::NONE;
     img_text(text, extents);
     img_patch_type(PatchType::OTHER);
-    img_text_layer(10000);
     transform_restore();
     if (interactive) {
         selectables.append(text.uuid, ObjectType::TEXT, text.placement.shift, extents.first, extents.second, 0,
@@ -639,8 +670,18 @@ void Canvas::render(const BusRipper &ripper)
     if (ripper.connection_count < 1) {
         draw_box(connector_pos, 0.25_mm, c);
     }
-    auto extents = draw_text0(connector_pos + Coordi(0, 0.5_mm), 1.5_mm, ripper.bus_member->name,
-                              ripper.mirror ? 32768 : 0, false, TextOrigin::BASELINE, c);
+    int angle = 0;
+    switch (ripper.orientation) {
+    case Orientation::LEFT:
+    case Orientation::DOWN:
+        angle = 32768;
+        break;
+
+    default:
+        angle = 0;
+    }
+    auto extents = draw_text0(connector_pos + Coordi(0, 0.5_mm), 1.5_mm, ripper.bus_member->name, angle, false,
+                              TextOrigin::BASELINE, c);
     if (!ripper.temp)
         targets.emplace(ripper.uuid, ObjectType::BUS_RIPPER, connector_pos);
     selectables.append(ripper.uuid, ObjectType::BUS_RIPPER, connector_pos, extents.first, extents.second);
@@ -651,11 +692,6 @@ void Canvas::render(const Warning &warn)
     draw_error(warn.position, 2e5, warn.text);
 }
 
-static const Coordf coordf_from_pt(const p2t::Point *pt)
-{
-    return Coordf(pt->x, pt->y);
-}
-
 static const Coordf coordf_from_pt(const TPPLPoint &p)
 {
     return Coordf(p.x, p.y);
@@ -674,43 +710,15 @@ void Canvas::render(const Polygon &ipoly, bool interactive)
         return;
     if (auto plane = dynamic_cast<Plane *>(poly.usage.ptr)) {
         triangle_type_current = Triangle::Type::PLANE;
+        auto tris = fragment_cache.get_triangles(*plane);
+        for (const auto &tri : tris) {
+            add_triangle(poly.layer, tri[0], tri[1], tri[2], ColorP::FROM_LAYER);
+        }
         for (const auto &frag : plane->fragments) {
-            std::vector<p2t::Point> point_store;
-            size_t pts_total = 0;
-            for (const auto &it : frag.paths)
-                pts_total += it.size();
-            point_store.reserve(pts_total); // important so that iterators won't
-                                            // get invalidated
-
-            std::vector<p2t::Point *> contour;
-            contour.reserve(frag.paths.front().size());
-            for (const auto &p : frag.paths.front()) {
-                point_store.emplace_back(p.X, p.Y);
-                contour.push_back(&point_store.back());
-            }
-            p2t::CDT cdt(contour);
-            for (size_t i = 1; i < frag.paths.size(); i++) {
-                auto &path = frag.paths.at(i);
-                std::vector<p2t::Point *> hole;
-                hole.reserve(path.size());
-                for (const auto &p : path) {
-                    point_store.emplace_back(p.X, p.Y);
-                    hole.push_back(&point_store.back());
-                }
-                cdt.AddHole(hole);
-            }
-            cdt.Triangulate();
-            auto tris = cdt.GetTriangles();
             ColorP co = ColorP::FROM_LAYER;
             if (frag.orphan == true)
-                co = ColorP::YELLOW;
+                co = ColorP::FRAG_ORPHAN;
 
-            for (const auto tri : tris) {
-                auto p0 = coordf_from_pt(tri->GetPoint(0));
-                auto p1 = coordf_from_pt(tri->GetPoint(1));
-                auto p2 = coordf_from_pt(tri->GetPoint(2));
-                add_triangle(poly.layer, p0, p1, p2, co);
-            }
             for (const auto &path : frag.paths) {
                 for (size_t i = 0; i < path.size(); i++) {
                     auto &c0 = path[i];
@@ -732,11 +740,13 @@ void Canvas::render(const Polygon &ipoly, bool interactive)
         TPPLPoly po;
         po.Init(poly.vertices.size());
         po.SetHole(false);
-        size_t i = 0;
-        for (auto &it : poly.vertices) {
-            po[i].x = it.position.x;
-            po[i].y = it.position.y;
-            i++;
+        {
+            size_t i = 0;
+            for (auto &it : poly.vertices) {
+                po[i].x = it.position.x;
+                po[i].y = it.position.y;
+                i++;
+            }
         }
         std::list<TPPLPoly> outpolys;
         TPPLPartition part;
@@ -763,8 +773,8 @@ void Canvas::render(const Polygon &ipoly, bool interactive)
             if (v_last) {
                 auto center = (v_last->position + it.position) / 2;
                 if (v_last->type != Polygon::Vertex::Type::ARC) {
-                    selectables.append(poly.uuid, ObjectType::POLYGON_EDGE, center, v_last->position, it.position,
-                                       i - 1, poly.layer);
+                    selectables.append_line(poly.uuid, ObjectType::POLYGON_EDGE, v_last->position, it.position, 0,
+                                            i - 1, poly.layer);
 
                     targets.emplace(poly.uuid, ObjectType::POLYGON_EDGE, center, i - 1, poly.layer);
                 }
@@ -781,8 +791,8 @@ void Canvas::render(const Polygon &ipoly, bool interactive)
         if (ipoly.vertices.back().type != Polygon::Vertex::Type::ARC) {
             auto center = (ipoly.vertices.front().position + ipoly.vertices.back().position) / 2;
             targets.emplace(poly.uuid, ObjectType::POLYGON_EDGE, center, i - 1, poly.layer);
-            selectables.append(poly.uuid, ObjectType::POLYGON_EDGE, center, ipoly.vertices.front().position,
-                               ipoly.vertices.back().position, i - 1, poly.layer);
+            selectables.append_line(poly.uuid, ObjectType::POLYGON_EDGE, poly.vertices.front().position,
+                                    ipoly.vertices.back().position, 0, i - 1, poly.layer);
         }
     }
 }
@@ -831,7 +841,7 @@ void Canvas::render(const Shape &shape, bool interactive)
 
 void Canvas::render(const Hole &hole, bool interactive)
 {
-    auto co = ColorP::WHITE;
+    auto co = ColorP::HOLE;
     img_hole(hole);
     if (img_mode)
         return;
@@ -841,9 +851,9 @@ void Canvas::render(const Hole &hole, bool interactive)
     int64_t d = hole.diameter / 2;
     int64_t l = std::max((int64_t)hole.length / 2 - d, (int64_t)0);
     if (hole.shape == Hole::Shape::ROUND) {
-        draw_arc(Coordi(), d, 0, 2 * M_PI, co);
+        draw_line(Coordf(), Coordf(100, 0), co, 10000, true, hole.diameter);
         if (hole.plated) {
-            draw_arc(Coordi(), 0.9 * d, 0, 2 * M_PI, co);
+            draw_line(Coordf(), Coordf(100, 0), co, 10000, true, hole.diameter * 0.9);
         }
         float x = hole.diameter / 2 / M_SQRT2;
         draw_line(Coordi(-x, -x), Coordi(x, x), co);
@@ -859,6 +869,8 @@ void Canvas::render(const Hole &hole, bool interactive)
         if (interactive)
             selectables.append(hole.uuid, ObjectType::HOLE, Coordi(), Coordi(-l - d, -d), Coordi(l + d, +d));
     }
+    if (interactive)
+        targets.emplace(hole.uuid, ObjectType::HOLE, hole.placement.shift);
     transform_restore();
 }
 
@@ -921,6 +933,9 @@ void Canvas::render(const Symbol &sym, bool on_sheet, bool smashed)
 
 void Canvas::render(const Sheet &sheet)
 {
+    if (sheet.frame.uuid) {
+        render(sheet.frame, true);
+    }
     for (const auto &it : sheet.junctions) {
         render(it.second, true, ObjectType::SCHEMATIC);
     }
@@ -948,26 +963,12 @@ void Canvas::render(const Sheet &sheet)
     for (const auto &it : sheet.bus_rippers) {
         render(it.second);
     }
-    render(sheet.frame);
-}
-
-void Canvas::render(const Frame &frame)
-{
-    if (frame.format == Frame::Format::NONE)
-        return;
-    int64_t width = frame.get_width();
-    int64_t height = frame.get_height();
-    int64_t border = frame.border;
-    Coordf bl(border, border);
-    Coordf br(width - border, border);
-    Coordf tr(width - border, height - border);
-    Coordf tl(border, height - border);
-
-    auto c = ColorP::FRAME;
-    draw_line(bl, br, c);
-    draw_line(br, tr, c);
-    draw_line(tr, tl, c);
-    draw_line(tl, bl, c);
+    for (const auto &it : sheet.lines) {
+        render(it.second);
+    }
+    for (const auto &it : sheet.arcs) {
+        render(it.second);
+    }
 }
 
 void Canvas::render(const Padstack &padstack, bool interactive)
@@ -998,38 +999,56 @@ void Canvas::render(const Package &pkg, bool interactive, bool smashed)
         }
     }
 
-    for (const auto &it : pkg.pads) {
-        transform_save();
-        transform.accumulate(it.second.placement);
-        auto bb = it.second.padstack.get_bbox(true); // only copper
-        auto a = bb.first;
-        auto b = bb.second;
+    if (!img_mode) {
+        for (const auto &it : pkg.pads) {
+            transform_save();
+            transform.accumulate(it.second.placement);
+            auto bb = it.second.padstack.get_bbox(true); // only copper
+            auto a = bb.first;
+            auto b = bb.second;
 
-        auto pad_width = abs(b.x - a.x);
-        auto pad_height = abs(b.y - a.y);
+            auto pad_width = abs(b.x - a.x);
+            auto pad_height = abs(b.y - a.y);
 
-        auto overlay_layer = get_overlay_layer(
-                it.second.padstack.type == Padstack::Type::TOP ? BoardLayers::TOP_COPPER : BoardLayers::BOTTOM_COPPER);
+            std::set<int> text_layers;
+            switch (it.second.padstack.type) {
+            case Padstack::Type::TOP:
+                text_layers.emplace(get_overlay_layer(BoardLayers::TOP_COPPER));
+                break;
 
-        set_lod_size(std::min(pad_height, pad_width));
-        if (it.second.net) {
-            draw_text_box(transform, pad_width, pad_height, it.second.name, ColorP::WHITE, overlay_layer, 0,
-                          TextBoxMode::UPPER);
-            draw_text_box(transform, pad_width, pad_height, it.second.net->name, ColorP::WHITE, overlay_layer, 0,
-                          TextBoxMode::LOWER);
+            case Padstack::Type::BOTTOM:
+                text_layers.emplace(get_overlay_layer(BoardLayers::BOTTOM_COPPER));
+                break;
+
+            default:
+                text_layers.emplace(get_overlay_layer(BoardLayers::TOP_COPPER));
+                text_layers.emplace(get_overlay_layer(BoardLayers::BOTTOM_COPPER));
+            }
+
+            set_lod_size(std::min(pad_height, pad_width));
+            for (const auto overlay_layer : text_layers) {
+                if (it.second.net && it.second.net->name.size() > 0) {
+                    draw_text_box(transform, pad_width, pad_height, it.second.name, ColorP::TEXT_OVERLAY, overlay_layer,
+                                  0, TextBoxMode::UPPER);
+                    draw_text_box(transform, pad_width, pad_height, it.second.net->name, ColorP::TEXT_OVERLAY,
+                                  overlay_layer, 0, TextBoxMode::LOWER);
+                }
+                else {
+                    draw_text_box(transform, pad_width, pad_height, it.second.name, ColorP::TEXT_OVERLAY, overlay_layer,
+                                  0, TextBoxMode::FULL);
+                }
+            }
+            set_lod_size(-1);
+            transform_restore();
         }
-        else {
-            draw_text_box(transform, pad_width, pad_height, it.second.name, ColorP::WHITE, overlay_layer, 0,
-                          TextBoxMode::FULL);
-        }
-        set_lod_size(-1);
-        transform_restore();
     }
+
     for (const auto &it : pkg.lines) {
         render(it.second, interactive);
     }
     for (const auto &it : pkg.texts) {
-        if (!smashed || !(it.second.layer == 20 || it.second.layer == 120))
+        if (!smashed
+            || !(it.second.layer == BoardLayers::TOP_SILKSCREEN || it.second.layer == BoardLayers::BOTTOM_SILKSCREEN))
             render(it.second, interactive);
     }
     for (const auto &it : pkg.arcs) {
@@ -1053,12 +1072,7 @@ void Canvas::render(const Package &pkg, bool interactive, bool smashed)
     }
 
     if (interactive) {
-        std::set<std::string> pad_names;
         for (const auto &it : pkg.pads) {
-            auto x = pad_names.insert(it.second.name);
-            if (!x.second) {
-                draw_error(it.second.placement.shift, 2e5, "duplicate pad name");
-            }
             transform_save();
             transform.accumulate(it.second.placement);
             auto bb = it.second.padstack.get_bbox();
@@ -1066,6 +1080,9 @@ void Canvas::render(const Package &pkg, bool interactive, bool smashed)
             transform_restore();
             targets.emplace(it.second.uuid, ObjectType::PAD, it.second.placement.shift);
         }
+        for (const auto &it : pkg.warnings) {
+            render(it);
+        }
     }
 }
 
@@ -1140,7 +1157,7 @@ void Canvas::render(const Via &via)
     if (via.locked) {
         auto ol = get_overlay_layer(BoardLayers::TOP_COPPER);
         draw_lock({0, 0}, 0.7 * std::min(std::abs(bb.second.x - bb.first.x), std::abs(bb.second.y - bb.first.y)),
-                  ColorP::WHITE, ol, true);
+                  ColorP::TEXT_OVERLAY, ol, true);
     }
     object_refs_current.pop_back();
     img_net(nullptr);
@@ -1186,21 +1203,22 @@ void Canvas::render(const class Dimension &dim)
     Coordd w = Coordd(-v.y, v.x);
     auto wn = w / sqrt(w.mag_sq());
 
+    ColorP co = ColorP::DIMENSION;
+
     if (v.mag_sq()) {
 
         auto q0 = p0 + wn * dim.label_distance;
         auto q1 = p1 + wn * dim.label_distance;
 
-        draw_line(dim.p0, p0 + wn * (dim.label_distance + sgn(dim.label_distance) * (int64_t)dim.label_size / 2),
-                  ColorP::WHITE, 10000, true, 0);
-        draw_line(dim.p1, p1 + wn * (dim.label_distance + sgn(dim.label_distance) * (int64_t)dim.label_size / 2),
-                  ColorP::WHITE, 10000, true, 0);
+        draw_line(dim.p0, p0 + wn * (dim.label_distance + sgn(dim.label_distance) * (int64_t)dim.label_size / 2), co,
+                  10000, true, 0);
+        draw_line(dim.p1, p1 + wn * (dim.label_distance + sgn(dim.label_distance) * (int64_t)dim.label_size / 2), co,
+                  10000, true, 0);
 
         auto length = sqrt(v.mag_sq());
         auto s = dim_to_string(length, false);
 
-        auto text_bb =
-                draw_text0({0, 0}, dim.label_size, s, 0, false, TextOrigin::CENTER, ColorP::WHITE, 10000, 0, false);
+        auto text_bb = draw_text0({0, 0}, dim.label_size, s, 0, false, TextOrigin::CENTER, co, 10000, 0, false);
         auto text_width = std::abs(text_bb.second.x - text_bb.first.x);
 
         Coordf text_pos = (q0 + v / 2) - (vn * text_width / 2);
@@ -1209,11 +1227,11 @@ void Canvas::render(const class Dimension &dim)
 
         if ((text_width + 2 * dim.label_size) > length) {
             text_pos = q1;
-            draw_line(q0, q1, ColorP::WHITE, 10000, true, 0);
+            draw_line(q0, q1, co, 10000, true, 0);
         }
         else {
-            draw_line(q0, (q0 + v / 2) - (vn * (text_width / 2 + .5 * dim.label_size)), ColorP::WHITE, 10000, true, 0);
-            draw_line(q1, (q0 + v / 2) + (vn * (text_width / 2 + .5 * dim.label_size)), ColorP::WHITE, 10000, true, 0);
+            draw_line(q0, (q0 + v / 2) - (vn * (text_width / 2 + .5 * dim.label_size)), co, 10000, true, 0);
+            draw_line(q1, (q0 + v / 2) + (vn * (text_width / 2 + .5 * dim.label_size)), co, 10000, true, 0);
         }
 
         float arrow_mul;
@@ -1228,18 +1246,17 @@ void Canvas::render(const class Dimension &dim)
         transform.reset();
         transform.shift = Coordi(q0.x, q0.y);
         transform.set_angle(angle_i);
-        draw_line({0, 0}, Coordf(arrow_mul * dim.label_size, dim.label_size / 2), ColorP::WHITE, 10000, true, 0);
-        draw_line({0, 0}, Coordf(arrow_mul * dim.label_size, (int64_t)dim.label_size / -2), ColorP::WHITE, 10000, true,
-                  0);
+        draw_line({0, 0}, Coordf(arrow_mul * dim.label_size, dim.label_size / 2), co, 10000, true, 0);
+        draw_line({0, 0}, Coordf(arrow_mul * dim.label_size, (int64_t)dim.label_size / -2), co, 10000, true, 0);
 
         transform.shift = Coordi(q1.x, q1.y);
-        draw_line({0, 0}, Coordf(-arrow_mul * dim.label_size, dim.label_size / 2), ColorP::WHITE, 10000, true, 0);
-        draw_line({0, 0}, Coordf(-arrow_mul * dim.label_size, (int64_t)dim.label_size / -2), ColorP::WHITE, 10000, true,
-                  0);
+        draw_line({0, 0}, Coordf(-arrow_mul * dim.label_size, dim.label_size / 2), co, 10000, true, 0);
+        draw_line({0, 0}, Coordf(-arrow_mul * dim.label_size, (int64_t)dim.label_size / -2), co, 10000, true, 0);
         transform_restore();
 
-        auto real_text_bb = draw_text0(Coordi(text_pos.x, text_pos.y), dim.label_size, s, angle_i, false,
-                                       TextOrigin::CENTER, ColorP::WHITE, 10000, 0, true);
+        auto real_text_bb = draw_text0(Coordi(text_pos.x, text_pos.y), dim.label_size, s,
+                                       get_flip_view() ? (32768 - angle_i) : angle_i, get_flip_view(),
+                                       TextOrigin::CENTER, co, 10000, 0, true);
 
         selectables.append(dim.uuid, ObjectType::DIMENSION, q0, real_text_bb.first, real_text_bb.second, 2);
         targets.emplace(dim.uuid, ObjectType::DIMENSION, Coordi(q0.x, q0.y), 2);
@@ -1318,5 +1335,39 @@ void Canvas::render(const Board &brd)
 
     clock_t end = clock();
     double elapsed_secs = double(end - begin) / CLOCKS_PER_SEC;
+    std::cout << "render took " << 1 / elapsed_secs << std::endl;
 }
+
+void Canvas::render(const Frame &fr, bool on_sheet)
+{
+    if (!on_sheet) {
+        for (const auto &it : fr.junctions) {
+            auto &junc = it.second;
+            selectables.append(junc.uuid, ObjectType::JUNCTION, junc.position, 0, 10000, true);
+            if (!junc.temp) {
+                targets.emplace(junc.uuid, ObjectType::JUNCTION, transform.transform(junc.position));
+            }
+        }
+    }
+    for (const auto &it : fr.lines) {
+        render(it.second, !on_sheet);
+    }
+
+
+    for (const auto &it : fr.arcs) {
+        render(it.second, !on_sheet);
+    }
+    for (const auto &it : fr.polygons) {
+        render(it.second, !on_sheet);
+    }
+    for (const auto &it : fr.texts) {
+        render(it.second, !on_sheet);
+    }
+    auto c = ColorP::FRAME;
+    draw_line(Coordf(), Coordf(fr.width, 0), c);
+    draw_line(Coordf(fr.width, 0), Coordf(fr.width, fr.height), c);
+    draw_line(Coordf(fr.width, fr.height), Coordf(0, fr.height), c);
+    draw_line(Coordf(0, fr.height), Coordf(), c);
+}
+
 } // namespace horizon
