@@ -7,6 +7,7 @@
 #include "logger/logger.hpp"
 #include "poly2tri/poly2tri.h"
 #include "util/step_importer.hpp"
+#include "pool/pool_manager.hpp"
 #include <cmath>
 #include <glm/glm.hpp>
 #include <glm/gtc/matrix_transform.hpp>
@@ -19,11 +20,12 @@ namespace horizon {
 
 Canvas3D::Canvas3D()
     : Gtk::GLArea(), CanvasPatch::CanvasPatch(), cover_renderer(this), wall_renderer(this), face_renderer(this),
-      background_renderer(this)
+      background_renderer(this), center(0)
 {
-    add_events(Gdk::BUTTON_PRESS_MASK | Gdk::BUTTON_RELEASE_MASK | Gdk::BUTTON_MOTION_MASK | Gdk::SCROLL_MASK
-               | Gdk::SMOOTH_SCROLL_MASK);
-
+    add_events(Gdk::BUTTON_PRESS_MASK | Gdk::BUTTON_RELEASE_MASK | Gdk::BUTTON_MOTION_MASK | Gdk::SCROLL_MASK);
+#ifndef G_OS_WIN32
+    add_events(Gdk::SMOOTH_SCROLL_MASK);
+#endif
     models_loading_dispatcher.connect([this] {
         package_height_max = 0;
         for (const auto &it : face_vertex_buffer) {
@@ -64,12 +66,19 @@ bool Canvas3D::on_motion_notify_event(GdkEventMotion *motion_event)
     auto delta = glm::mat2(1, 0, 0, -1) * (glm::vec2(motion_event->x, motion_event->y) - pointer_pos_orig);
     if (pan_mode == PanMode::ROTATE) {
         cam_azimuth = cam_azimuth_orig - (delta.x / width) * 360;
-        cam_elevation = CLAMP(cam_elevation_orig - (delta.y / height) * 90, -89.9, 89.9);
+        cam_elevation = cam_elevation_orig - (delta.y / height) * 90;
+        while (cam_elevation >= 360)
+            cam_elevation -= 360;
+        while (cam_elevation < 0)
+            cam_elevation += 360;
+        if (cam_elevation > 180)
+            cam_elevation -= 360;
         queue_draw();
     }
     else if (pan_mode == PanMode::MOVE) {
         center = center_orig
-                 + glm::rotate(glm::mat2(1, 0, 0, sin(glm::radians(cam_elevation))) * delta * 0.1f,
+                 + glm::rotate(glm::mat2(1, 0, 0, sin(glm::radians(cam_elevation))) * delta * 0.1f * cam_distance
+                                       / 105.f,
                                glm::radians(cam_azimuth - 90));
         queue_draw();
     }
@@ -82,25 +91,84 @@ bool Canvas3D::on_button_release_event(GdkEventButton *button_event)
     return Gtk::GLArea::on_button_release_event(button_event);
 }
 
+int Canvas3D::_animate_step(GdkFrameClock *frame_clock)
+{
+    auto r = zoom_animator.step(gdk_frame_clock_get_frame_time(frame_clock) / 1e6);
+    if (!r) { // should stop
+        return G_SOURCE_REMOVE;
+    }
+    auto s = zoom_animator.get_s();
+
+    cam_distance = zoom_animation_cam_dist_orig * pow(1.5, s);
+    queue_draw();
+
+    if (std::abs((s - zoom_animator.target) / std::max(std::abs(zoom_animator.target), 1.f)) < .005) {
+        cam_distance = zoom_animation_cam_dist_orig * pow(1.5, zoom_animator.target);
+        queue_draw();
+        zoom_animator.stop();
+        return G_SOURCE_REMOVE;
+    }
+
+    return G_SOURCE_CONTINUE;
+}
+
+static int tick_cb(GtkWidget *cwidget, GdkFrameClock *frame_clock, gpointer user_data)
+{
+    Gtk::Widget *widget = Glib::wrap(cwidget);
+    auto canvas = dynamic_cast<Canvas3D *>(widget);
+    return canvas->_animate_step(frame_clock);
+}
+
+
 bool Canvas3D::on_scroll_event(GdkEventScroll *scroll_event)
 {
-    float dist_new = cam_distance;
+
+    float inc = 0;
+    float factor = 1;
+    if (scroll_event->state & Gdk::SHIFT_MASK)
+        factor = .5;
     if (scroll_event->direction == GDK_SCROLL_UP) {
-        dist_new = cam_distance / 1.5;
+        inc = -1;
     }
     else if (scroll_event->direction == GDK_SCROLL_DOWN) {
-        dist_new = cam_distance * 1.5;
+        inc = 1;
     }
     else if (scroll_event->direction == GDK_SCROLL_SMOOTH) {
         gdouble sx, sy;
         gdk_event_get_scroll_deltas((GdkEvent *)scroll_event, &sx, &sy);
-        dist_new = cam_distance * powf(1.5, sy);
+        inc = sy;
     }
-    cam_distance = std::max(0.f, dist_new);
-    queue_draw();
+    inc *= factor;
+    if (smooth_zoom) {
+        if (inc == 0)
+            return Gtk::GLArea::on_scroll_event(scroll_event);
+        if (!zoom_animator.is_running()) {
+            zoom_animator.start();
+            zoom_animation_cam_dist_orig = cam_distance;
+            gtk_widget_add_tick_callback(GTK_WIDGET(gobj()), &tick_cb, nullptr, nullptr);
+        }
+        zoom_animator.target += inc;
+    }
+    else {
+        cam_distance *= pow(1.5, inc);
+        queue_draw();
+    }
+
+
     return Gtk::GLArea::on_scroll_event(scroll_event);
 }
 
+void Canvas3D::inc_cam_azimuth(float v)
+{
+    cam_azimuth += v;
+    while (cam_azimuth < 0)
+        cam_azimuth += 360;
+
+    while (cam_azimuth > 360)
+        cam_azimuth -= 360;
+    queue_draw();
+}
+
 void Canvas3D::request_push()
 {
     needs_push = true;
@@ -260,14 +328,12 @@ void Canvas3D::prepare()
     int layer = BoardLayers::TOP_COPPER;
     layers[layer].offset = 0;
     layers[layer].thickness = brd->stackup.at(0).thickness / 1e6;
-    layers[layer].color = {1, .8, 0};
     layers[layer].explode_mul = 1;
     prepare_layer(layer);
 
     layer = BoardLayers::BOTTOM_COPPER;
     layers[layer].offset = -board_thickness;
     layers[layer].thickness = +(brd->stackup.at(layer).thickness / 1e6);
-    layers[layer].color = {1, .8, 0};
     layers[layer].explode_mul = -2 * n_inner_layers - 1;
     prepare_layer(layer);
 
@@ -277,72 +343,71 @@ void Canvas3D::prepare()
             layer = -i - 1;
             layers[layer].offset = offset;
             layers[layer].thickness = -(brd->stackup.at(layer).thickness / 1e6);
-            layers[layer].color = {1, .8, 0};
             layers[layer].explode_mul = -1 - 2 * i;
             offset -= brd->stackup.at(layer).thickness / 1e6 + brd->stackup.at(layer).substrate_thickness / 1e6;
             prepare_layer(layer);
         }
     }
 
-    if (show_substrate) {
-        layer = BoardLayers::L_OUTLINE;
-        layers[layer].offset = 0;
-        layers[layer].thickness = -(brd->stackup.at(0).substrate_thickness / 1e6);
-        layers[layer].explode_mul = 0;
-        layers[layer].color = {.2, .15, 0};
-        prepare_layer(layer);
+    layer = BoardLayers::L_OUTLINE;
+    layers[layer].offset = 0;
+    layers[layer].thickness = -(brd->stackup.at(0).substrate_thickness / 1e6);
+    layers[layer].explode_mul = 0;
+    prepare_layer(layer);
 
-        float offset = -(brd->stackup.at(0).substrate_thickness / 1e6);
-        for (int i = 0; i < n_inner_layers; i++) {
-            int l = 10000 + i;
-            offset -= brd->stackup.at(-i - 1).thickness / 1e6;
-            layers[l] = layers[layer];
-            layers[l].offset = offset;
-            layers[l].thickness = -(brd->stackup.at(-i - 1).substrate_thickness / 1e6);
-            layers[l].explode_mul = -2 - 2 * i;
+    float offset = -(brd->stackup.at(0).substrate_thickness / 1e6);
+    for (int i = 0; i < n_inner_layers; i++) {
+        int l = 10000 + i;
+        offset -= brd->stackup.at(-i - 1).thickness / 1e6;
+        layers[l] = layers[layer];
+        layers[l].offset = offset;
+        layers[l].thickness = -(brd->stackup.at(-i - 1).substrate_thickness / 1e6);
+        layers[l].explode_mul = -2 - 2 * i;
 
-            offset -= brd->stackup.at(-i - 1).substrate_thickness / 1e6;
-        }
+        offset -= brd->stackup.at(-i - 1).substrate_thickness / 1e6;
     }
 
-    if (show_solder_mask) {
-        layer = BoardLayers::TOP_MASK;
-        layers[layer].offset = .036;
-        layers[layer].thickness = 0.035;
-        layers[layer].color = solder_mask_color;
-        layers[layer].alpha = .8;
-        layers[layer].explode_mul = 2;
-        prepare_soldermask(layer);
+    layer = BoardLayers::TOP_MASK;
+    layers[layer].offset = brd->stackup.at(0).thickness / 1e6 + 1e-3;
+    layers[layer].thickness = 0.01;
+    layers[layer].alpha = .8;
+    layers[layer].explode_mul = 3;
+    prepare_soldermask(layer);
 
-        layer = BoardLayers::BOTTOM_MASK;
-        layers[layer].offset = -board_thickness - .036;
-        layers[layer].thickness = 0.035;
-        layers[layer].color = solder_mask_color;
-        layers[layer].alpha = .8;
-        layers[layer].explode_mul = -2 * n_inner_layers - 2;
-        prepare_soldermask(layer);
-    }
+    layer = BoardLayers::BOTTOM_MASK;
+    layers[layer].offset = -board_thickness - 1e-3;
+    layers[layer].thickness = 0.035;
+    layers[layer].alpha = .8;
+    layers[layer].explode_mul = -2 * n_inner_layers - 3;
+    prepare_soldermask(layer);
 
-    if (show_silkscreen) {
-        layer = BoardLayers::TOP_SILKSCREEN;
-        layers[layer].offset = .07;
-        layers[layer].thickness = 0.035;
-        layers[layer].color = {1, 1, 1};
-        layers[layer].explode_mul = 3;
-        prepare_layer(layer);
+    layer = BoardLayers::TOP_SILKSCREEN;
+    layers[layer].offset = brd->stackup.at(0).thickness / 1e6 + 1e-3;
+    layers[layer].thickness = 0.035;
+    layers[layer].explode_mul = 4;
+    prepare_layer(layer);
 
-        layer = BoardLayers::BOTTOM_SILKSCREEN;
-        layers[layer].offset = -board_thickness - .07;
-        layers[layer].thickness = -0.035;
-        layers[layer].color = {1, 1, 1};
-        layers[layer].explode_mul = -2 * n_inner_layers - 3;
-        prepare_layer(layer);
-    }
+    layer = BoardLayers::BOTTOM_SILKSCREEN;
+    layers[layer].offset = -board_thickness - .1e-3;
+    layers[layer].thickness = -0.035;
+    layers[layer].explode_mul = -2 * n_inner_layers - 4;
+    prepare_layer(layer);
+
+    layer = BoardLayers::TOP_PASTE;
+    layers[layer].offset = brd->stackup.at(0).thickness / 1e6 + 1e-3;
+    layers[layer].thickness = 0.035;
+    layers[layer].explode_mul = 2;
+    prepare_layer(layer);
+
+    layer = BoardLayers::BOTTOM_PASTE;
+    layers[layer].offset = -board_thickness;
+    layers[layer].thickness = -0.035;
+    layers[layer].explode_mul = -2 * n_inner_layers - 2;
+    prepare_layer(layer);
 
     layer = 20000; // pth holes
     layers[layer].offset = 0;
     layers[layer].thickness = -board_thickness;
-    layers[layer].color = {1, .8, 0};
     for (const auto &it : patches) {
         if (it.first.layer == 10000 && it.first.type == PatchType::HOLE_PTH) {
             ClipperLib::ClipperOffset ofs;
@@ -425,12 +490,22 @@ float Canvas3D::get_layer_offset(int layer)
     return layers[layer].offset + layers[layer].explode_mul * explode;
 }
 
-void Canvas3D::load_3d_model(const std::string &filename, const std::string &base_path)
+float Canvas3D::get_layer_thickness(int layer) const
+{
+    if (layer == BoardLayers::L_OUTLINE && explode == 0) {
+        return layers.at(BoardLayers::BOTTOM_COPPER).offset + layers.at(BoardLayers::BOTTOM_COPPER).thickness;
+    }
+    else {
+        return layers.at(layer).thickness;
+    }
+}
+
+void Canvas3D::load_3d_model(const std::string &filename, const std::string &filename_abs)
 {
     if (models.count(filename))
         return;
 
-    auto faces = STEPImporter::import(Glib::build_filename(base_path, filename));
+    auto faces = STEPImporter::import(filename_abs);
     // canvas->face_vertex_buffer.reserve(faces.size());
     size_t vertex_offset = face_vertex_buffer.size();
     size_t first_index = face_index_buffer.size();
@@ -452,25 +527,36 @@ void Canvas3D::load_3d_model(const std::string &filename, const std::string &bas
                    std::forward_as_tuple(first_index, last_index - first_index));
 }
 
-void Canvas3D::load_models_thread(std::set<std::string> model_filenames, std::string base_path)
+void Canvas3D::load_models_thread(std::map<std::string, std::string> model_filenames)
 {
     std::lock_guard<std::mutex> lock(models_loading_mutex);
     for (const auto &it : model_filenames) {
-        load_3d_model(it, base_path);
+        load_3d_model(it.first, it.second);
     }
     models_loading_dispatcher.emit();
 }
 
 void Canvas3D::load_models_async(Pool *pool)
 {
-    std::set<std::string> model_filenames;
+    std::map<std::string, std::string> model_filenames; // first: relative, second: absolute
     for (const auto &it : brd->packages) {
         auto model = it.second.package.get_model(it.second.model);
-        if (model)
-            model_filenames.insert(model->filename);
+        if (model) {
+            std::string model_filename;
+            if (it.second.pool_package == pool->get_package(it.second.pool_package->uuid)) {
+                // package is from pool, ask pool for model filename (might come from cache)
+                model_filename = pool->get_model_filename(it.second.pool_package->uuid, model->uuid);
+            }
+            else {
+                // package is not from pool (from package editor, use filename relative to current pool)
+                model_filename = Glib::build_filename(pool->get_base_path(), model->filename);
+            }
+            if (model_filename.size())
+                model_filenames[model->filename] = model_filename;
+        }
     }
     s_signal_models_loading.emit(true);
-    std::thread thr(&Canvas3D::load_models_thread, this, model_filenames, pool->get_base_path());
+    std::thread thr(&Canvas3D::load_models_thread, this, model_filenames);
 
     thr.detach();
 }
@@ -482,8 +568,22 @@ void Canvas3D::clear_3d_models()
     models.clear();
 }
 
+void Canvas3D::update_packages()
+{
+    prepare_packages();
+    request_push();
+}
+
+void Canvas3D::set_highlights(const std::set<UUID> &pkgs)
+{
+    packages_highlight = pkgs;
+    update_packages();
+}
+
 void Canvas3D::prepare_packages()
 {
+    if (!brd)
+        return;
     package_transform_idxs.clear();
     package_transforms.clear();
     std::map<std::string, std::set<const BoardPackage *>> pkg_map;
@@ -498,7 +598,8 @@ void Canvas3D::prepare_packages()
         for (const auto &it_brd_pkg : it_pkg.second) {
             const auto &pl = it_brd_pkg->placement;
             const auto &pkg = it_brd_pkg->package;
-            package_transforms.emplace_back(pl.shift.x / 1e6, pl.shift.y / 1e6, pl.get_angle(), it_brd_pkg->flip);
+            package_transforms.emplace_back(pl.shift.x / 1e6, pl.shift.y / 1e6, pl.get_angle(), it_brd_pkg->flip,
+                                            packages_highlight.count(it_brd_pkg->uuid));
             auto &tr = package_transforms.back();
             const auto model = pkg.get_model(it_brd_pkg->model);
             if (model) {
@@ -522,6 +623,57 @@ void Canvas3D::set_msaa(unsigned int samples)
     queue_draw();
 }
 
+bool Canvas3D::layer_is_visible(int layer) const
+{
+    if (layer == 20000) // pth holes
+        return true;
+
+    if (layer == BoardLayers::TOP_MASK || layer == BoardLayers::BOTTOM_MASK)
+        return show_solder_mask;
+
+    if (layer == BoardLayers::TOP_PASTE || layer == BoardLayers::BOTTOM_PASTE)
+        return show_solder_paste;
+
+    if (layer == BoardLayers::TOP_SILKSCREEN || layer == BoardLayers::BOTTOM_SILKSCREEN)
+        return show_silkscreen;
+
+    if (layer == BoardLayers::L_OUTLINE || layer >= 10000) {
+        if (show_substrate) {
+            if (layer == BoardLayers::L_OUTLINE)
+                return true;
+            else {
+                return explode > 0;
+            }
+        }
+        else {
+            return false;
+        }
+    }
+    if (layer < BoardLayers::TOP_COPPER && layer > BoardLayers::BOTTOM_COPPER)
+        return show_substrate == false || explode > 0;
+
+    return true;
+}
+
+Color Canvas3D::get_layer_color(int layer) const
+{
+    if (layer == 20000 || BoardLayers::is_copper(layer)) // pth or cu
+        return {1, .8, 0};
+
+    if (layer == BoardLayers::TOP_MASK || layer == BoardLayers::BOTTOM_MASK)
+        return solder_mask_color;
+
+    if (layer == BoardLayers::TOP_PASTE || layer == BoardLayers::BOTTOM_PASTE)
+        return {.7, .7, .7};
+
+    if (layer == BoardLayers::TOP_SILKSCREEN || layer == BoardLayers::BOTTOM_SILKSCREEN)
+        return {1, 1, 1};
+
+    if (layer == BoardLayers::L_OUTLINE || layer >= 10000)
+        return substrate_color;
+    return {1, 0, 0};
+}
+
 bool Canvas3D::on_render(const Glib::RefPtr<Gdk::GLContext> &context)
 {
     if (needs_push) {
@@ -559,7 +711,7 @@ bool Canvas3D::on_render(const Glib::RefPtr<Gdk::GLContext> &context)
 
     glm::vec3 right(sin(phi - 3.14f / 2.0f), cos(phi - 3.14f / 2.0f), 0);
 
-    viewmat = glm::lookAt(cam_pos, glm::vec3(center, 0), glm::vec3(0, 0, 1));
+    viewmat = glm::lookAt(cam_pos, glm::vec3(center, 0), glm::vec3(0, 0, std::abs(cam_elevation) < 90 ? 1 : -1));
 
     float cam_dist_min = std::max(std::abs(cam_pos.z) - (10 + explode * (brd->get_n_inner_layers() * 2 + 3)), 1.0f);
     float cam_dist_max = 0;
@@ -577,8 +729,14 @@ bool Canvas3D::on_render(const Glib::RefPtr<Gdk::GLContext> &context)
         cam_dist_max = std::max(dist, cam_dist_max);
         cam_dist_min = std::min(dist, cam_dist_min);
     }
-
-    projmat = glm::perspective(glm::radians(cam_fov), width / height, cam_dist_min, cam_dist_max);
+    float m = 0.000475 * cam_distance;
+    float d = cam_dist_max * 2;
+    if (projection == Projection::PERSP) {
+        projmat = glm::perspective(glm::radians(cam_fov), width / height, cam_dist_min / 2, cam_dist_max * 2);
+    }
+    else {
+        projmat = glm::ortho(-width * m, width * m, -height * m, height * m, -d, d);
+    }
 
     cam_normal = glm::normalize(cam_offset);
     wall_renderer.render();
