@@ -16,6 +16,8 @@
 #include "schematic/net_label.hpp"
 #include "schematic/power_symbol.hpp"
 #include "schematic/schematic_symbol.hpp"
+#include "board/via.hpp"
+#include "board/track.hpp"
 #include "util/uuid.hpp"
 #include <map>
 #include <set>
@@ -42,6 +44,8 @@ public:
     std::map<UUID, Shape> shapes;
     std::map<UUID, PowerSymbol> power_symbols;
     std::map<UUID, NetLabel> net_labels;
+    std::map<UUID, Via> vias;
+    std::map<UUID, Track> tracks;
 
     json serialize();
 
