@@ -52,6 +52,15 @@
 #include "tool_smash.hpp"
 #include "tool_update_all_planes.hpp"
 #include "tool_generate_courtyard.hpp"
+#include "tool_set_group.hpp"
+#include "tool_copy_placement.hpp"
+#include "tool_copy_tracks.hpp"
+#include "tool_swap_nets.hpp"
+#include "tool_line_loop_to_polygon.hpp"
+#include "tool_change_unit.hpp"
+#include "tool_set_nc_all.hpp"
+#include "tool_set_nc.hpp"
+#include "tool_add_keepout.hpp"
 
 #include "common/dimension.hpp"
 #include "logger/logger.hpp"
@@ -69,6 +78,11 @@ std::unique_ptr<ToolBase> Core::create_tool(ToolID tool_id)
     case ToolID::MIRROR_X:
     case ToolID::MIRROR_Y:
     case ToolID::ROTATE:
+    case ToolID::MOVE_KEY:
+    case ToolID::MOVE_KEY_UP:
+    case ToolID::MOVE_KEY_DOWN:
+    case ToolID::MOVE_KEY_LEFT:
+    case ToolID::MOVE_KEY_RIGHT:
         return std::make_unique<ToolMove>(this, tool_id);
 
     case ToolID::PLACE_JUNCTION:
@@ -130,6 +144,7 @@ std::unique_ptr<ToolBase> Core::create_tool(ToolID tool_id)
     case ToolID::ANNOTATE:
     case ToolID::EDIT_SCHEMATIC_PROPERTIES:
     case ToolID::MANAGE_POWER_NETS:
+    case ToolID::EDIT_FRAME_PROPERTIES:
         return std::make_unique<ToolManageBuses>(this, tool_id);
 
     case ToolID::DRAW_POLYGON:
@@ -204,12 +219,16 @@ std::unique_ptr<ToolBase> Core::create_tool(ToolID tool_id)
     case ToolID::ROUTE_TRACK_INTERACTIVE:
     case ToolID::ROUTE_DIFFPAIR_INTERACTIVE:
     case ToolID::DRAG_TRACK_INTERACTIVE:
+    case ToolID::TUNE_TRACK:
+    case ToolID::TUNE_DIFFPAIR:
+    case ToolID::TUNE_DIFFPAIR_SKEW:
         return std::make_unique<ToolRouteTrackInteractive>(this, tool_id);
 
     case ToolID::EDIT_VIA:
         return std::make_unique<ToolEditVia>(this, tool_id);
 
     case ToolID::ROTATE_ARBITRARY:
+    case ToolID::SCALE:
         return std::make_unique<ToolRotateArbitrary>(this, tool_id);
 
     case ToolID::ADD_PLANE:
@@ -232,6 +251,7 @@ std::unique_ptr<ToolBase> Core::create_tool(ToolID tool_id)
         return std::make_unique<ToolSetDiffpair>(this, tool_id);
 
     case ToolID::SELECT_MORE:
+    case ToolID::SELECT_MORE_NO_VIA:
         return std::make_unique<ToolSelectMore>(this, tool_id);
 
     case ToolID::SET_VIA_NET:
@@ -255,6 +275,39 @@ std::unique_ptr<ToolBase> Core::create_tool(ToolID tool_id)
     case ToolID::GENERATE_COURTYARD:
         return std::make_unique<ToolGenerateCourtyard>(this, tool_id);
 
+    case ToolID::SET_GROUP:
+    case ToolID::CLEAR_GROUP:
+    case ToolID::SET_TAG:
+    case ToolID::CLEAR_TAG:
+        return std::make_unique<ToolSetGroup>(this, tool_id);
+
+    case ToolID::COPY_PLACEMENT:
+        return std::make_unique<ToolCopyPlacement>(this, tool_id);
+
+    case ToolID::COPY_TRACKS:
+        return std::make_unique<ToolCopyTracks>(this, tool_id);
+
+    case ToolID::SWAP_NETS:
+        return std::make_unique<ToolSwapNets>(this, tool_id);
+
+    case ToolID::LINE_LOOP_TO_POLYGON:
+        return std::make_unique<ToolLineLoopToPolygon>(this, tool_id);
+
+    case ToolID::CHANGE_UNIT:
+        return std::make_unique<ToolChangeUnit>(this, tool_id);
+
+    case ToolID::SET_ALL_NC:
+    case ToolID::CLEAR_ALL_NC:
+        return std::make_unique<ToolSetNotConnectedAll>(this, tool_id);
+
+    case ToolID::SET_NC:
+    case ToolID::CLEAR_NC:
+        return std::make_unique<ToolSetNotConnected>(this, tool_id);
+
+    case ToolID::ADD_KEEPOUT:
+    case ToolID::EDIT_KEEPOUT:
+        return std::make_unique<ToolAddKeepout>(this, tool_id);
+
     default:
         return nullptr;
     }
@@ -269,6 +322,15 @@ ToolResponse Core::tool_begin(ToolID tool_id, const ToolArgs &args, class ImpInt
     update_rules(); // write rules to board, so tool has the current rules
     try {
         tool = create_tool(tool_id);
+        {
+            auto sp = tool->get_settings_proxy();
+            if (sp != nullptr) {
+                auto tid = tool->get_tool_id_for_settings();
+                auto j = s_signal_load_tool_settings.emit(tid);
+                if (j != nullptr)
+                    sp->load_from_json(j);
+            }
+        }
         tool->set_imp_interface(imp);
         if (transient)
             tool->set_transient();
@@ -298,6 +360,10 @@ ToolResponse Core::tool_begin(ToolID tool_id, const ToolArgs &args, class ImpInt
             return ToolResponse::end();
         }
         if (r.end_tool) {
+            auto tid = tool->get_tool_id_for_settings();
+            auto settings = tool->get_settings_const();
+            if (settings)
+                s_signal_save_tool_settings.emit(tid, settings->serialize());
             tool.reset();
             s_signal_tool_changed.emit(ToolID::NONE);
             rebuild();
@@ -342,6 +408,10 @@ ToolResponse Core::tool_update(const ToolArgs &args)
             return ToolResponse::end();
         }
         if (r.end_tool) {
+            auto tid = tool->get_tool_id_for_settings();
+            auto settings = tool->get_settings_const();
+            if (settings)
+                s_signal_save_tool_settings.emit(tid, settings->serialize());
             tool.reset();
             s_signal_tool_changed.emit(ToolID::NONE);
             rebuild();
@@ -508,6 +578,38 @@ void Core::delete_dimension(const UUID &uu)
     map->erase(uu);
 }
 
+Keepout *Core::insert_keepout(const UUID &uu)
+{
+    auto map = get_keepout_map();
+    auto x = map->emplace(std::make_pair(uu, uu));
+    return &(x.first->second);
+}
+
+Keepout *Core::get_keepout(const UUID &uu)
+{
+    auto map = get_keepout_map();
+    return &map->at(uu);
+}
+
+void Core::delete_keepout(const UUID &uu)
+{
+    auto map = get_keepout_map();
+    map->erase(uu);
+}
+
+std::vector<Keepout *> Core::get_keepouts()
+{
+    auto *map = get_keepout_map();
+    std::vector<Keepout *> r;
+    if (!map)
+        return r;
+    for (auto &it : *map) {
+        r.push_back(&it.second);
+    }
+    return r;
+}
+
+
 void Core::rebuild(bool from_undo)
 {
     if (!from_undo && !reverted) {
@@ -546,6 +648,13 @@ void Core::redo()
     set_needs_save();
 }
 
+void Core::history_clear()
+{
+    history.clear();
+    history_current = -1;
+    signal_can_undo_redo().emit();
+}
+
 bool Core::can_redo() const
 {
     return history_current + 1 != (int)history.size();
@@ -616,4 +725,9 @@ void ToolBase::set_transient()
 {
     is_transient = true;
 }
+
+ToolSettingsProxy::~ToolSettingsProxy()
+{
+    tool->apply_settings();
+}
 } // namespace horizon
