@@ -11,8 +11,8 @@ CoreBoard::CoreBoard(const std::string &board_filename, const std::string &block
                      Pool &pool)
     : via_padstack_provider(via_dir, pool), block(Block::new_from_file(block_filename, pool)),
       brd(Board::new_from_file(board_filename, block, pool, via_padstack_provider)), rules(brd.rules),
-      fab_output_settings(brd.fab_output_settings), m_board_filename(board_filename), m_block_filename(block_filename),
-      m_via_dir(via_dir)
+      fab_output_settings(brd.fab_output_settings), colors(brd.colors), m_board_filename(board_filename),
+      m_block_filename(block_filename), m_via_dir(via_dir)
 {
     m_pool = &pool;
     rebuild();
@@ -76,6 +76,7 @@ bool CoreBoard::has_object_type(ObjectType ty)
     case ObjectType::BOARD_PACKAGE:
     case ObjectType::VIA:
     case ObjectType::DIMENSION:
+    case ObjectType::KEEPOUT:
         return true;
         break;
     default:;
@@ -108,6 +109,10 @@ std::map<UUID, Dimension> *CoreBoard::get_dimension_map()
 {
     return &brd.dimensions;
 }
+std::map<UUID, Keepout> *CoreBoard::get_keepout_map()
+{
+    return &brd.keepouts;
+}
 
 bool CoreBoard::get_property(ObjectType type, const UUID &uu, ObjectProperty::ID property, PropertyValue &value)
 {
@@ -309,9 +314,14 @@ bool CoreBoard::set_property(ObjectType type, const UUID &uu, ObjectProperty::ID
 
         case ObjectProperty::ID::POSITION_X:
         case ObjectProperty::ID::POSITION_Y:
-        case ObjectProperty::ID::ANGLE:
+        case ObjectProperty::ID::ANGLE: {
+            const auto shift_before = pkg->placement.shift;
             set_placement(pkg->placement, value, property);
-            break;
+            auto delta = pkg->placement.shift - shift_before;
+            for (auto &text : pkg->texts) {
+                text->placement.shift += delta;
+            }
+        } break;
 
         default:
             return false;
@@ -496,8 +506,12 @@ std::vector<Line *> CoreBoard::get_lines(bool work)
 
 void CoreBoard::rebuild(bool from_undo)
 {
+    clock_t begin = clock();
     brd.expand();
     Core::rebuild(from_undo);
+    clock_t end = clock();
+    double elapsed_secs = double(end - begin) / CLOCKS_PER_SEC;
+    std::cout << "rebuild took " << elapsed_secs << std::endl;
 }
 
 LayerProvider *CoreBoard::get_layer_provider()
@@ -546,21 +560,42 @@ void CoreBoard::revert()
     reverted = true;
 }
 
+CoreBoard::HistoryItem::HistoryItem(const Block &b, const Board &r) : block(b), brd(r.serialize())
+{
+}
+
 void CoreBoard::history_push()
 {
     history.push_back(std::make_unique<CoreBoard::HistoryItem>(block, brd));
     auto x = dynamic_cast<CoreBoard::HistoryItem *>(history.back().get());
-    x->brd.block = &x->block;
-    x->brd.update_refs();
+    for (const auto &it : brd.planes) {
+        auto &frags = x->fragments[it.first];
+        frags.reserve(it.second.fragments.size());
+        std::copy(it.second.fragments.begin(), it.second.fragments.end(), std::back_inserter(frags));
+    }
 }
 
 void CoreBoard::history_load(unsigned int i)
 {
     auto x = dynamic_cast<CoreBoard::HistoryItem *>(history.at(history_current).get());
-    brd = x->brd;
+    UUID uu(x->brd.at("uuid").get<std::string>());
+    std::map<UUID, unsigned int> plane_revs;
+    for (const auto &it : brd.planes) {
+        plane_revs[it.first] = it.second.revision;
+    }
+    brd.~Board(); // reconstruct board from json
+    new (&brd) Board(uu, x->brd, x->block, *m_pool, via_padstack_provider);
+
     block = x->block;
-    brd.block = &block;
     brd.update_refs();
+    for (const auto &it : x->fragments) {
+        if (brd.planes.count(it.first)) {
+            std::copy(it.second.begin(), it.second.end(), std::back_inserter(brd.planes.at(it.first).fragments));
+            if (plane_revs.count(it.first))
+                brd.planes.at(it.first).revision = plane_revs.at(it.first) + 1;
+        }
+    }
+    brd.expand();
 }
 
 json CoreBoard::get_meta()
@@ -600,6 +635,7 @@ void CoreBoard::save()
 {
     brd.rules = rules;
     brd.fab_output_settings = fab_output_settings;
+    brd.colors = colors;
     auto j = brd.serialize();
     auto save_meta = s_signal_request_save_meta.emit();
     j["_imp"] = save_meta;
