@@ -1,9 +1,10 @@
 #pragma once
 #include "core.hpp"
+#include "tool_helper_line_width_setting.hpp"
 
 namespace horizon {
 
-class ToolDrawLine : public ToolBase {
+class ToolDrawLine : public ToolHelperLineWidthSetting {
 public:
     ToolDrawLine(Core *c, ToolID tid);
     ToolResponse begin(const ToolArgs &args) override;
@@ -14,10 +15,16 @@ public:
         return true;
     }
 
+    void apply_settings() override;
+
 private:
     Junction *temp_junc = 0;
     Line *temp_line = 0;
     void update_tip();
     bool first_line = true;
+    std::set<const Junction *> junctions_created;
+    enum class Mode { X, Y, ARB };
+    Mode mode = Mode::ARB;
+    void do_move(const Coordi &c);
 };
 } // namespace horizon
