@@ -37,6 +37,7 @@ void ToolDrawLineNet::restart(const Coordi &c)
     temp_line_head = nullptr;
     temp_line_mid = nullptr;
     component_floating = nullptr;
+    net_label = nullptr;
     update_tip();
 }
 
@@ -73,6 +74,9 @@ void ToolDrawLineNet::update_tip()
     ss << "<b>LMB:</b>place junction/connect <b>RMB:</b>finish and delete last "
           "segment <b>space:</b>place junction <b>⏎:</b>finish <b>/:</b>line "
           "posture <b>a:</b>arbitrary angle ";
+    if (net_label) {
+        ss << "<b>+-:</b>change label size ";
+    }
     if (temp_line_head) {
         if (temp_line_head->net) {
             ss << "<b>b:</b>toggle label <b>r:</b>rotate label";
@@ -141,6 +145,10 @@ ToolResponse ToolDrawLineNet::update(const ToolArgs &args)
 
                     if (sym->component->connections.count(connpath)) { // pin is connected
                         Connection &conn = sym->component->connections.at(connpath);
+                        if (conn.net == nullptr) {
+                            imp->tool_bar_flash("can't connect to NC pin");
+                            return ToolResponse();
+                        }
                         net = conn.net;
                     }
                     else { // pin needs net
@@ -213,6 +221,10 @@ ToolResponse ToolDrawLineNet::update(const ToolArgs &args)
                     }
                     if (sym->component->connections.count(connpath)) { // pin is connected
                         Connection &conn = sym->component->connections.at(connpath);
+                        if (conn.net == nullptr) {
+                            imp->tool_bar_flash("can't connect to NC pin");
+                            return ToolResponse();
+                        }
                         if (temp_line_head->net && (temp_line_head->net->uuid != conn.net->uuid)) { // has net
                             if (merge_nets(conn.net, temp_line_head->net)) {
                                 return ToolResponse();
@@ -258,6 +270,10 @@ ToolResponse ToolDrawLineNet::update(const ToolArgs &args)
                         restart(args.coords);
                         return ToolResponse();
                     }
+                    if (net_label) {
+                        restart(args.coords);
+                        return ToolResponse();
+                    }
 
                     ju = temp_junc_head;
                     for (auto it : core.c->get_net_lines()) {
@@ -407,10 +423,21 @@ ToolResponse ToolDrawLineNet::update(const ToolArgs &args)
                 }
             }
         }
+        else if (args.key == GDK_KEY_plus || args.key == GDK_KEY_equal) {
+            if (net_label)
+                net_label->size += 0.5_mm;
+        }
+        else if (args.key == GDK_KEY_minus) {
+            if (net_label) {
+                if (net_label->size > 0.5_mm) {
+                    net_label->size -= 0.5_mm;
+                }
+            }
+        }
         else if (args.key == GDK_KEY_r) {
             if (net_label)
                 net_label->orientation =
-                        ToolHelperMove::transform_orienation(net_label->orientation, args.key == GDK_KEY_r);
+                        ToolHelperMove::transform_orientation(net_label->orientation, args.key == GDK_KEY_r);
         }
         else if (args.key == GDK_KEY_a) {
             bend_mode = BendMode::ARB;
