@@ -15,6 +15,10 @@ public:
     {
         return true;
     }
+    bool handles_esc() override
+    {
+        return true;
+    }
 
 private:
     Coordi last;
@@ -27,5 +31,15 @@ private:
     Mode mode = Mode::ARB;
     Coordi get_coord(const Coordi &c);
     void do_move(const Coordi &c);
+
+    void collect_nets();
+    std::set<UUID> nets;
+
+    bool update_airwires = true;
+    void finish();
+    bool is_key = false;
+    Coordi key_delta;
+
+    std::set<class Plane *> planes;
 };
 } // namespace horizon
