@@ -21,5 +21,7 @@ private:
     void apply_shift(Coordi &c, const Coordi &cursor_pos);
     Coordi shift;
     bool shift_set = false;
+    bool data_received = false;
+    ToolResponse begin_paste(const std::string &paste_data, const Coordi &cursor_pos);
 };
 } // namespace horizon
