@@ -1,5 +1,4 @@
 #pragma once
-#include "core/core.hpp"
 #include "action.hpp"
 #include "common/lut.hpp"
 #include <map>
@@ -13,16 +12,23 @@ public:
         AVAILABLE_IN_PADSTACK = (1 << 2),
         AVAILABLE_IN_PACKAGE = (1 << 3),
         AVAILABLE_IN_BOARD = (1 << 4),
+        AVAILABLE_IN_FRAME = (1 << 5),
         AVAILABLE_EVERYWHERE = 0xff,
         AVAILABLE_LAYERED = AVAILABLE_IN_PACKAGE | AVAILABLE_IN_PADSTACK | AVAILABLE_IN_BOARD,
         AVAILABLE_IN_PACKAGE_AND_BOARD = AVAILABLE_IN_PACKAGE | AVAILABLE_IN_BOARD,
         AVAILABLE_IN_SCHEMATIC_AND_BOARD = AVAILABLE_IN_SCHEMATIC | AVAILABLE_IN_BOARD
     };
 
-    enum Flags { FLAGS_DEFAULT = 0, FLAGS_IN_TOOL = (1 << 1), FLAGS_NO_POPOVER = (1 << 2) };
+    enum Flags {
+        FLAGS_DEFAULT = 0,
+        FLAGS_IN_TOOL = (1 << 1),
+        FLAGS_NO_POPOVER = (1 << 2),
+        FLAGS_NO_MENU = (1 << 3),
+        FLAGS_SPECIFIC = (1 << 4)
+    };
 
-    ActionCatalogItem(const std::string &n, ActionGroup gr, Availability av, Flags fl = FLAGS_DEFAULT)
-        : name(n), group(gr), flags(fl), availability(av){};
+    ActionCatalogItem(const std::string &n, ActionGroup gr, int av, int fl = FLAGS_DEFAULT)
+        : name(n), group(gr), flags(static_cast<Flags>(fl)), availability(static_cast<Availability>(av)){};
 
     const std::string name;
     ActionGroup group;
