@@ -6,7 +6,6 @@
 #include "logger/logger.hpp"
 #include "pool/part.hpp"
 #include "pool/pool_cached.hpp"
-#include "preferences_window.hpp"
 #include "property_panels/property_panels.hpp"
 #include "rules/rules_window.hpp"
 #include "util/gtk_util.hpp"
@@ -16,7 +15,11 @@
 #include "widgets/spin_button_dim.hpp"
 #include "action_catalog.hpp"
 #include "tool_popover.hpp"
+#include "hud_util.hpp"
+#include "util/str_util.hpp"
+#include "parameter_window.hpp"
 #include <glibmm/main.h>
+#include <glibmm/markup.h>
 #include <gtkmm.h>
 #include <iomanip>
 #include <functional>
@@ -54,6 +57,12 @@ ImpBase::ImpBase(const PoolParams &params)
         chan = Glib::IOChannel::create_from_fd(fd);
 #endif
 
+        {
+            auto pid_p = Glib::getenv("HORIZON_MGR_PID");
+            if (pid_p.size())
+                mgr_pid = std::stoi(pid_p);
+        }
+
         Glib::signal_io().connect(
                 [this](Glib::IOCondition cond) {
                     while (sock_broadcast_rx.getsockopt<int>(ZMQ_EVENTS) & ZMQ_POLLIN) {
@@ -67,11 +76,12 @@ ImpBase::ImpBase(const PoolParams &params)
                             handle_broadcast(j);
                         }
                     }
+                    preferences.signal_changed().emit();
                     return true;
                 },
                 chan, Glib::IO_IN | Glib::IO_HUP);
     }
-    auto ep_project = Glib::getenv("HORIZON_EP_PROJECT");
+    auto ep_project = Glib::getenv("HORIZON_EP_MGR");
     if (ep_project.size()) {
         sock_project.connect(ep_project);
     }
@@ -105,50 +115,44 @@ bool ImpBase::handle_close(GdkEventAny *ev)
         return false;
 
     if (!core.r->get_needs_save()) {
-        if (preferences_window)
-            preferences_window->hide();
         return false;
     }
+    if (!read_only) {
+        Gtk::MessageDialog md(*main_window, "Save changes before closing?", false /* use_markup */,
+                              Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_NONE);
+        md.set_secondary_text("If you don't save, all your changes will be permanently lost.");
+        md.add_button("Close without Saving", 1);
+        md.add_button("Cancel", Gtk::RESPONSE_CANCEL);
+        md.add_button("Save", 2);
+        switch (md.run()) {
+        case 1:
+            return false; // close
 
-    Gtk::MessageDialog md(*main_window, "Save changes before closing?", false /* use_markup */, Gtk::MESSAGE_QUESTION,
-                          Gtk::BUTTONS_NONE);
-    md.set_secondary_text("If you don't save, all your changes will be permanently lost.");
-    md.add_button("Close without Saving", 1);
-    md.add_button("Cancel", Gtk::RESPONSE_CANCEL);
-    md.add_button("Save", 2);
-    switch (md.run()) {
-    case 1:
-        if (preferences_window)
-            preferences_window->hide();
-        return false; // close
+        case 2:
+            core.r->save();
+            return false; // close
 
-    case 2:
-        core.r->save();
-        return false; // close
+        default:
+            return true; // keep window open
+        }
+    }
+    else {
+        Gtk::MessageDialog md(*main_window, "Document is read only", false /* use_markup */, Gtk::MESSAGE_QUESTION,
+                              Gtk::BUTTONS_NONE);
+        md.set_secondary_text("You won't be able to save your changes");
+        md.add_button("Close without Saving", 1);
+        md.add_button("Cancel", Gtk::RESPONSE_CANCEL);
+        switch (md.run()) {
+        case 1:
+            return false; // close
 
-    default:
-        return true; // keep window open
+        default:
+            return true; // keep window open
+        }
     }
     return false;
 }
 
-void ImpBase::show_preferences_window()
-{
-    if (!preferences_window) {
-        preferences.load();
-        preferences_window = new ImpPreferencesWindow(&preferences);
-        preferences_window->set_transient_for(*main_window);
-
-        preferences_window->signal_hide().connect([this] {
-            delete preferences_window;
-            preferences_window = nullptr;
-            preferences.save();
-            preferences.unlock();
-        });
-    }
-    preferences_window->present();
-}
-
 #define GET_WIDGET(name)                                                                                               \
     do {                                                                                                               \
         main_window->builder->get_widget(#name, name);                                                                 \
@@ -189,6 +193,15 @@ void ImpBase::update_selection_label()
     main_window->selection_label->set_text(la);
 }
 
+std::string ImpBase::get_tool_settings_filename(ToolID id)
+{
+    auto type_str = object_type_lut.lookup_reverse(get_editor_type());
+    auto settings_dir = Glib::build_filename(get_config_dir(), "tool_settings", type_str);
+    if (!Glib::file_test(settings_dir, Glib::FILE_TEST_IS_DIR))
+        Gio::File::create_for_path(settings_dir)->make_directory_with_parents();
+    return Glib::build_filename(settings_dir, tool_lut.lookup_reverse(id) + ".json");
+}
+
 void ImpBase::run(int argc, char *argv[])
 {
     auto app = Gtk::Application::create(argc, argv, "net.carrotIndustries.horizon.Imp", Gio::APPLICATION_NON_UNIQUE);
@@ -226,7 +239,7 @@ void ImpBase::run(int argc, char *argv[])
                 {CanvasGL::SelectionQualifier::INCLUDE_ORIGIN, selection_qualifier_include_origin_button},
                 {CanvasGL::SelectionQualifier::AUTO, selection_qualifier_auto_button}};
         bind_widget<CanvasGL::SelectionQualifier>(qual_map, canvas->selection_qualifier,
-                                                  [this](auto v) { update_selection_label(); });
+                                                  [this](auto v) { this->update_selection_label(); });
 
         std::map<CanvasGL::SelectionTool, Gtk::RadioButton *> tool_map = {
                 {CanvasGL::SelectionTool::BOX, selection_tool_box_button},
@@ -237,7 +250,7 @@ void ImpBase::run(int argc, char *argv[])
                                              [this, selection_qualifier_touch_box_button, selection_qualifier_box,
                                               selection_qualifier_auto_button,
                                               selection_qualifier_include_origin_button](auto v) {
-                                                 update_selection_label();
+                                                 this->update_selection_label();
                                                  auto is_paint = (v == CanvasGL::SelectionTool::PAINT);
                                                  if (is_paint) {
                                                      selection_qualifier_touch_box_button->set_active(true);
@@ -312,14 +325,22 @@ void ImpBase::run(int argc, char *argv[])
     key_sequence_dialog = std::make_unique<KeySequenceDialog>(this->main_window);
 
     connect_action(ActionID::SELECTION_FILTER, [this](const auto &a) { selection_filter_dialog->present(); });
-    connect_action(ActionID::SAVE, [this](const auto &a) { core.r->save(); });
+    connect_action(ActionID::SAVE, [this](const auto &a) {
+        if (!read_only)
+            core.r->save();
+    });
     connect_action(ActionID::UNDO, [this](const auto &a) {
         core.r->undo();
-        canvas_update_from_pp();
+        this->canvas_update_from_pp();
     });
     connect_action(ActionID::REDO, [this](const auto &a) {
         core.r->redo();
-        canvas_update_from_pp();
+        this->canvas_update_from_pp();
+    });
+
+    connect_action(ActionID::RELOAD_POOL, [this](const auto &a) {
+        core.r->reload_pool();
+        this->canvas_update_from_pp();
     });
 
     connect_action(ActionID::COPY,
@@ -327,7 +348,7 @@ void ImpBase::run(int argc, char *argv[])
 
     connect_action(ActionID::DUPLICATE, [this](const auto &a) {
         clipboard->copy(canvas->get_selection(), canvas->get_cursor_pos());
-        tool_begin(ToolID::PASTE);
+        this->tool_begin(ToolID::PASTE);
     });
 
     connect_action(ActionID::HELP, [this](const auto &a) { key_sequence_dialog->show(); });
@@ -344,7 +365,7 @@ void ImpBase::run(int argc, char *argv[])
         rect.set_y(c.y);
         tool_popover->set_pointing_to(rect);
 
-        update_action_sensitivity();
+        this->update_action_sensitivity();
         std::map<std::pair<ActionID, ToolID>, bool> can_begin;
         auto sel = canvas->get_selection();
         for (const auto &it : action_catalog) {
@@ -353,7 +374,7 @@ void ImpBase::run(int argc, char *argv[])
                 can_begin[it.first] = r;
             }
             else {
-                can_begin[it.first] = get_action_sensitive(it.first);
+                can_begin[it.first] = this->get_action_sensitive(it.first);
             }
         }
         tool_popover->set_can_begin(can_begin);
@@ -366,21 +387,10 @@ void ImpBase::run(int argc, char *argv[])
     });
 
     connect_action(ActionID::PREFERENCES, [this](const auto &a) {
-        if (preferences.lock()) {
-            show_preferences_window();
-        }
-        else {
-            Gtk::MessageDialog md(*main_window, "Can't lock preferences", false /* use_markup */, Gtk::MESSAGE_ERROR,
-                                  Gtk::BUTTONS_NONE);
-            md.add_button("OK", Gtk::RESPONSE_OK);
-            md.add_button("Force unlock", 1);
-            md.set_secondary_text("Close all other preferences dialogs first");
-            if (md.run() == 1) {
-                preferences.unlock();
-                preferences.lock();
-                show_preferences_window();
-            }
-        }
+        json j;
+        j["op"] = "preferences";
+        allow_set_foreground_window(mgr_pid);
+        this->send_json(j);
     });
 
     for (const auto &it : action_catalog) {
@@ -389,6 +399,21 @@ void ImpBase::run(int argc, char *argv[])
         }
     }
 
+    connect_action(ActionID::VIEW_TOP, [this](const auto &a) {
+        canvas->set_flip_view(false);
+        this->canvas_update_from_pp();
+    });
+
+    connect_action(ActionID::VIEW_BOTTOM, [this](const auto &a) {
+        canvas->set_flip_view(true);
+        this->canvas_update_from_pp();
+    });
+
+    connect_action(ActionID::FLIP_VIEW, [this](const auto &a) {
+        canvas->set_flip_view(!canvas->get_flip_view());
+        this->canvas_update_from_pp();
+    });
+
 
     grid_spin_button = Gtk::manage(new SpinButtonDim());
     grid_spin_button->set_range(0.1_mm, 10_mm);
@@ -401,7 +426,14 @@ void ImpBase::run(int argc, char *argv[])
     auto save_button = create_action_button(make_action(ActionID::SAVE));
     save_button->show();
     main_window->header->pack_start(*save_button);
-    core.r->signal_needs_save().connect([this](bool v) { update_action_sensitivity(); });
+    core.r->signal_needs_save().connect([this](bool v) {
+        update_action_sensitivity();
+        json j;
+        j["op"] = "needs-save";
+        j["pid"] = getpid();
+        j["needs_save"] = core.r->get_needs_save();
+        send_json(j);
+    });
     set_action_sensitive(make_action(ActionID::SAVE), false);
 
     {
@@ -409,13 +441,13 @@ void ImpBase::run(int argc, char *argv[])
         undo_redo_box->get_style_context()->add_class("linked");
 
         auto undo_button = create_action_button(make_action(ActionID::UNDO));
-        undo_button->set_label("");
+        gtk_button_set_label(undo_button->gobj(), NULL);
         undo_button->set_tooltip_text("Undo");
         undo_button->set_image_from_icon_name("edit-undo-symbolic", Gtk::ICON_SIZE_BUTTON);
         undo_redo_box->pack_start(*undo_button, false, false, 0);
 
         auto redo_button = create_action_button(make_action(ActionID::REDO));
-        redo_button->set_label("");
+        gtk_button_set_label(redo_button->gobj(), NULL);
         redo_button->set_tooltip_text("Redo");
         redo_button->set_image_from_icon_name("edit-redo-symbolic", Gtk::ICON_SIZE_BUTTON);
         undo_redo_box->pack_start(*redo_button, false, false, 0);
@@ -428,6 +460,22 @@ void ImpBase::run(int argc, char *argv[])
     canvas->signal_selection_changed().connect([this] { update_action_sensitivity(); });
     core.r->signal_can_undo_redo().emit();
 
+    core.r->signal_load_tool_settings().connect([this](ToolID id) {
+        json j;
+        auto fn = get_tool_settings_filename(id);
+        if (Glib::file_test(fn, Glib::FILE_TEST_IS_REGULAR)) {
+            std::ifstream ifs(fn);
+            if (!ifs.is_open()) {
+                return j;
+            }
+            ifs >> j;
+            ifs.close();
+        }
+        return j;
+    });
+    core.r->signal_save_tool_settings().connect(
+            [this](ToolID id, json j) { save_json_to_file(get_tool_settings_filename(id), j); });
+
     auto help_button = create_action_button(make_action(ActionID::HELP));
     help_button->show();
     main_window->header->pack_end(*help_button);
@@ -458,21 +506,6 @@ void ImpBase::run(int argc, char *argv[])
             [this](ActionID action_id, ToolID tool_id) { trigger_action(std::make_pair(action_id, tool_id)); });
 
 
-    preferences.signal_changed().connect(sigc::mem_fun(this, &ImpBase::apply_settings));
-
-    preferences.load();
-
-    preferences_monitor = Gio::File::create_for_path(ImpPreferences::get_preferences_filename())->monitor();
-
-    preferences_monitor->signal_changed().connect([this](const Glib::RefPtr<Gio::File> &file,
-                                                         const Glib::RefPtr<Gio::File> &file_other,
-                                                         Gio::FileMonitorEvent ev) {
-        if (ev == Gio::FILE_MONITOR_EVENT_CHANGES_DONE_HINT)
-            preferences.load();
-    });
-
-    main_window->add_action("preferences", [this] { trigger_action(ActionID::PREFERENCES); });
-
     log_window = new LogWindow(main_window);
     Logger::get().set_log_handler([this](const Logger::Item &it) { log_window->get_view()->push_log(it); });
 
@@ -481,12 +514,28 @@ void ImpBase::run(int argc, char *argv[])
     main_window->signal_delete_event().connect(sigc::mem_fun(this, &ImpBase::handle_close));
 
     for (const auto &la : core.r->get_layer_provider()->get_layers()) {
-        canvas->set_layer_display(la.first, LayerDisplay(true, LayerDisplay::Mode::FILL, la.second.color));
+        canvas->set_layer_display(la.first, LayerDisplay(true, LayerDisplay::Mode::FILL));
     }
 
     construct();
 
-    apply_settings();
+    preferences.signal_changed().connect(sigc::mem_fun(this, &ImpBase::apply_preferences));
+
+    preferences.load();
+
+    preferences_monitor = Gio::File::create_for_path(Preferences::get_preferences_filename())->monitor();
+
+    preferences_monitor->signal_changed().connect([this](const Glib::RefPtr<Gio::File> &file,
+                                                         const Glib::RefPtr<Gio::File> &file_other,
+                                                         Gio::FileMonitorEvent ev) {
+        if (ev == Gio::FILE_MONITOR_EVENT_CHANGES_DONE_HINT)
+            preferences.load();
+    });
+
+    if (sockets_connected)
+        main_window->add_action("preferences", [this] { trigger_action(ActionID::PREFERENCES); });
+
+    apply_preferences();
 
     canvas->property_work_layer().signal_changed().connect([this] {
         if (core.r->tool_is_active()) {
@@ -508,6 +557,9 @@ void ImpBase::run(int argc, char *argv[])
 
     core.r->signal_tool_changed().connect([this](ToolID id) { s_signal_action_sensitive.emit(); });
 
+    canvas->signal_hover_selection_changed().connect(sigc::mem_fun(this, &ImpBase::hud_update));
+    canvas->signal_selection_changed().connect(sigc::mem_fun(this, &ImpBase::hud_update));
+
     canvas_update();
 
     auto bbox = core.r->get_bbox();
@@ -548,9 +600,132 @@ void ImpBase::run(int argc, char *argv[])
 
     sc();
 
+    core.r->signal_rebuilt().connect([this] { update_monitor(); });
+
     app->run(*main_window);
 }
 
+void ImpBase::parameter_window_add_polygon_expand(ParameterWindow *parameter_window)
+{
+    auto button = Gtk::manage(new Gtk::Button("Polygon expand"));
+    parameter_window->add_button(button);
+    button->signal_clicked().connect([this, parameter_window] {
+        auto sel = canvas->get_selection();
+        if (sel.size() == 1) {
+            auto &s = *sel.begin();
+            if (s.type == ObjectType::POLYGON_EDGE || s.type == ObjectType::POLYGON_VERTEX) {
+                auto poly = core.r->get_polygon(s.uuid);
+                if (!poly->has_arcs()) {
+                    std::stringstream ss;
+                    ss.imbue(std::locale("C"));
+                    ss << "expand-polygon [ " << poly->parameter_class << " ";
+                    for (const auto &it : poly->vertices) {
+                        ss << it.position.x << " " << it.position.y << " ";
+                    }
+                    ss << "]\n";
+                    parameter_window->insert_text(ss.str());
+                }
+            }
+        }
+    });
+}
+
+void ImpBase::hud_update()
+{
+    if (core.r->tool_is_active())
+        return;
+
+    auto sel = canvas->get_selection();
+    if (sel.size()) {
+        std::string hud_text = get_hud_text(sel);
+        trim(hud_text);
+        hud_text += ImpBase::get_hud_text(sel);
+        trim(hud_text);
+        main_window->hud_update(hud_text);
+    }
+    else {
+        main_window->hud_update("");
+    }
+}
+
+std::string ImpBase::get_hud_text(std::set<SelectableRef> &sel)
+{
+    std::string s;
+    if (sel_count_type(sel, ObjectType::LINE)) {
+        auto n = sel_count_type(sel, ObjectType::LINE);
+        s += "\n\n<b>" + std::to_string(n) + " " + object_descriptions.at(ObjectType::LINE).get_name_for_n(n)
+             + "</b>\n";
+        std::set<int> layers;
+        int64_t length = 0;
+        for (const auto &it : sel) {
+            if (it.type == ObjectType::LINE) {
+                const auto li = core.r->get_line(it.uuid);
+                layers.insert(li->layer);
+                length += sqrt((li->from->position - li->to->position).mag_sq());
+            }
+        }
+        s += "Layers: ";
+        for (auto layer : layers) {
+            s += core.r->get_layer_provider()->get_layers().at(layer).name + " ";
+        }
+        s += "\nTotal length: " + dim_to_string(length, false);
+        sel_erase_type(sel, ObjectType::LINE);
+        return s;
+    }
+    trim(s);
+    if (sel.size()) {
+        s += "\n\n<b>Others:</b>\n";
+        for (const auto &it : object_descriptions) {
+            auto n = std::count_if(sel.begin(), sel.end(), [&it](const auto &a) { return a.type == it.first; });
+            if (n) {
+                s += std::to_string(n) + " " + it.second.get_name_for_n(n) + "\n";
+            }
+        }
+        sel.clear();
+    }
+    return s;
+}
+
+std::string ImpBase::get_hud_text_for_net(const Net *net)
+{
+    if (!net)
+        return "No net";
+
+    std::string s = "Net: " + net->name + "\n";
+    s += "Net class " + net->net_class->name + "\n";
+    if (net->is_power)
+        s += "is power net";
+
+    trim(s);
+    return s;
+}
+
+std::string ImpBase::get_hud_text_for_part(const Part *part)
+{
+    if (!part)
+        return "No part";
+
+    std::string s = "MPN: " + part->get_MPN() + "\n";
+    s += "Manufacturer: " + part->get_manufacturer() + "\n";
+    if (part->get_description().size())
+        s += part->get_description() + "\n";
+    if (part->get_datasheet().size())
+        s += "<a href=\"" + Glib::Markup::escape_text(part->get_datasheet()) + "\" title=\""
+             + Glib::Markup::escape_text(Glib::Markup::escape_text(part->get_datasheet())) + "\">Datasheet</a>\n";
+
+    trim(s);
+    return s;
+}
+
+void ImpBase::edit_pool_item(ObjectType type, const UUID &uu)
+{
+    json j;
+    j["op"] = "edit";
+    j["type"] = object_type_lut.lookup_reverse(type);
+    j["uuid"] = static_cast<std::string>(uu);
+    send_json(j);
+}
+
 Gtk::Button *ImpBase::create_action_button(std::pair<ActionID, ToolID> action)
 {
     auto &catitem = action_catalog.at(action);
@@ -593,6 +768,8 @@ void ImpBase::handle_drag()
     auto delta = pos - cursor_pos_drag_begin;
     if (delta.mag_sq() > (50 * 50)) {
         {
+            highlights.clear();
+            update_highlights();
             ToolArgs args;
             args.coords = cursor_pos_grid_drag_begin;
             args.selection = selection_for_drag_move;
@@ -603,32 +780,11 @@ void ImpBase::handle_drag()
     }
 }
 
-void ImpBase::apply_settings()
+void ImpBase::apply_preferences()
 {
     auto canvas_prefs = get_canvas_preferences();
-    if (canvas_prefs->background_color == CanvasPreferences::BackgroundColor::BLUE) {
-        canvas->set_background_color(Color::new_from_int(0, 24, 64));
-        canvas->set_grid_color(Color::new_from_int(0, 78, 208));
-    }
-    else {
-        canvas->set_grid_color({1, 1, 1});
-        canvas->set_background_color({0, 0, 0});
-    }
-    canvas->set_grid_style(canvas_prefs->grid_style);
-    canvas->set_grid_alpha(canvas_prefs->grid_opacity);
-    canvas->set_highlight_dim(canvas_prefs->highlight_dim);
-    canvas->set_highlight_shadow(canvas_prefs->highlight_shadow);
-    canvas->set_highlight_lighten(canvas_prefs->highlight_lighten);
-    switch (canvas_prefs->grid_fine_modifier) {
-    case CanvasPreferences::GridFineModifier::ALT:
-        canvas->grid_fine_modifier = Gdk::MOD1_MASK;
-        break;
-    case CanvasPreferences::GridFineModifier::CTRL:
-        canvas->grid_fine_modifier = Gdk::CONTROL_MASK;
-        break;
-    }
+    canvas->set_appearance(canvas_prefs->appearance);
     canvas->show_all_junctions_in_schematic = preferences.schematic.show_all_junctions;
-    canvas->set_msaa(canvas_prefs->msaa);
 
     auto av = get_editor_type_for_action();
     for (auto &it : action_connections) {
@@ -657,6 +813,7 @@ void ImpBase::apply_settings()
             tool_popover->set_key_sequences(it.first, it.second.key_sequences);
         }
     }
+    canvas->smooth_zoom = preferences.zoom.smooth_zoom_2d;
 }
 
 void ImpBase::canvas_update_from_pp()
@@ -699,11 +856,18 @@ ActionConnection &ImpBase::connect_action(ActionID action_id, ToolID tool_id,
     return act;
 }
 
-void ImpBase::tool_begin(ToolID id)
+void ImpBase::tool_begin(ToolID id, bool override_selection, const std::set<SelectableRef> &sel)
 {
+    highlights.clear();
+    update_highlights();
     ToolArgs args;
     args.coords = canvas->get_cursor_pos();
-    args.selection = canvas->get_selection();
+
+    if (override_selection)
+        args.selection = sel;
+    else
+        args.selection = canvas->get_selection();
+
     args.work_layer = canvas->property_work_layer();
     ToolResponse r = core.r->tool_begin(id, args, imp_interface.get());
     tool_process(r);
@@ -744,7 +908,7 @@ bool ImpBase::get_action_sensitive(std::pair<ActionID, ToolID> action) const
 
 void ImpBase::update_action_sensitivity()
 {
-    set_action_sensitive(make_action(ActionID::SAVE), core.r->get_needs_save());
+    set_action_sensitive(make_action(ActionID::SAVE), !read_only && core.r->get_needs_save());
     set_action_sensitive(make_action(ActionID::UNDO), core.r->can_undo());
     set_action_sensitive(make_action(ActionID::REDO), core.r->can_redo());
     auto sel = canvas->get_selection();
@@ -858,7 +1022,10 @@ bool ImpBase::handle_action_key(GdkEventKey *ev)
         for (auto &it : action_connections) {
             for (const auto &it2 : it.second.key_sequences) {
                 auto minl = std::min(keys_current.size(), it2.size());
-                if (minl && std::equal(keys_current.begin(), keys_current.begin() + minl, it2.begin())) {
+                if (minl && std::equal(keys_current.begin(), keys_current.begin() + minl, it2.begin())
+                    && !(core.r->tool_is_active()
+                         && !(action_catalog.at(std::make_pair(it.second.action_id, it.second.tool_id)).flags
+                              & ActionCatalogItem::FLAGS_IN_TOOL))) {
                     connections_matched.insert(&it.second);
                 }
             }
@@ -932,8 +1099,61 @@ bool ImpBase::handle_click_release(GdkEventButton *button_event)
     return false;
 }
 
+void ImpBase::create_context_menu(Gtk::Menu *parent, const std::set<SelectableRef> &sel)
+{
+    update_action_sensitivity();
+    Gtk::SeparatorMenuItem *sep = nullptr;
+    for (const auto &it_gr : action_group_catalog) {
+        bool have_item = false;
+        for (const auto &it : action_catalog) {
+            if (it.second.group == it_gr.first && (it.second.availability & get_editor_type_for_action())
+                && !(it.second.flags & ActionCatalogItem::FLAGS_NO_MENU)) {
+                if (it.first.first == ActionID::TOOL) {
+                    auto r = core.r->tool_can_begin(it.first.second, {sel});
+                    if (r.first && r.second) {
+                        auto la_sub = Gtk::manage(new Gtk::MenuItem(it.second.name));
+                        ToolID tool_id = it.first.second;
+                        std::set<SelectableRef> sr(sel);
+                        la_sub->signal_activate().connect([this, tool_id, sr] {
+                            canvas->set_selection(sr, false);
+                            fix_cursor_pos();
+                            tool_begin(tool_id);
+                        });
+                        la_sub->show();
+                        parent->append(*la_sub);
+                        have_item = true;
+                    }
+                }
+                else {
+                    if (get_action_sensitive(it.first) && (it.second.flags & ActionCatalogItem::FLAGS_SPECIFIC)) {
+                        auto la_sub = Gtk::manage(new Gtk::MenuItem(it.second.name));
+                        ActionID action_id = it.first.first;
+                        std::set<SelectableRef> sr(sel);
+                        la_sub->signal_activate().connect([this, action_id, sr] {
+                            canvas->set_selection(sr, false);
+                            fix_cursor_pos();
+                            trigger_action(make_action(action_id));
+                        });
+                        la_sub->show();
+                        parent->append(*la_sub);
+                        have_item = true;
+                    }
+                }
+            }
+        }
+        if (have_item) {
+            sep = Gtk::manage(new Gtk::SeparatorMenuItem);
+            sep->show();
+            parent->append(*sep);
+        }
+    }
+    if (sep)
+        delete sep;
+}
+
 bool ImpBase::handle_click(GdkEventButton *button_event)
 {
+    bool need_menu = false;
     if (core.r->tool_is_active() && button_event->button != 2 && !(button_event->state & Gdk::SHIFT_MASK)) {
         ToolArgs args;
         args.type = ToolEventType::CLICK;
@@ -986,107 +1206,29 @@ bool ImpBase::handle_click(GdkEventButton *button_event)
                     la->signal_deselect().connect([this] { canvas->set_selection({}, false); });
                     auto submenu = Gtk::manage(new Gtk::Menu);
 
-                    {
-                        auto la2 = Gtk::manage(new Gtk::MenuItem("Copy"));
-                        la2->signal_activate().connect([this, sr] {
-                            canvas->set_selection({sr}, false);
-                            clipboard->copy(canvas->get_selection(), canvas->get_cursor_pos());
-                        });
-                        la2->show();
-                        submenu->append(*la2);
-                    }
-                    {
-                        auto la2 = Gtk::manage(new Gtk::MenuItem("Duplicate"));
-                        la2->signal_activate().connect([this, sr] {
-                            canvas->set_selection({sr}, false);
-                            clipboard->copy(canvas->get_selection(), canvas->get_cursor_pos());
-                            tool_begin(ToolID::PASTE);
-                        });
-                        la2->show();
-                        submenu->append(*la2);
-                    }
-
-                    {
-                        auto sep = Gtk::manage(new Gtk::SeparatorMenuItem);
-                        sep->show();
-                        submenu->append(*sep);
-                    }
-
-                    for (const auto &it : action_catalog) {
-                        if (it.first.first == ActionID::TOOL) {
-                            auto r = core.r->tool_can_begin(it.first.second, {sr});
-                            if (r.first && r.second) {
-                                auto la_sub = Gtk::manage(new Gtk::MenuItem(it.second.name));
-                                ToolID tool_id = it.first.second;
-                                la_sub->signal_activate().connect([this, tool_id, sr] {
-                                    canvas->set_selection({sr}, false);
-                                    fix_cursor_pos();
-                                    tool_begin(tool_id);
-                                });
-                                la_sub->show();
-                                submenu->append(*la_sub);
-                            }
-                        }
-                    }
+                    create_context_menu(submenu, {sr});
                     la->set_submenu(*submenu);
                     la->show();
                     context_menu->append(*la);
                 }
-#if GTK_CHECK_VERSION(3, 22, 0)
-                context_menu->popup_at_pointer((GdkEvent *)button_event);
-#else
-                context_menu->popup(0, gtk_get_current_event_time());
-#endif
+                need_menu = true;
                 sel_for_menu.clear();
             }
         }
+
         if (sel_for_menu.size()) {
-
-            {
-                auto la = Gtk::manage(new Gtk::MenuItem("Copy"));
-                la->signal_activate().connect(
-                        [this] { clipboard->copy(canvas->get_selection(), canvas->get_cursor_pos()); });
-                la->show();
-                context_menu->append(*la);
-            }
-            {
-                auto la = Gtk::manage(new Gtk::MenuItem("Duplicate"));
-                la->signal_activate().connect([this] {
-                    clipboard->copy(canvas->get_selection(), canvas->get_cursor_pos());
-                    tool_begin(ToolID::PASTE);
-                });
-                la->show();
-                context_menu->append(*la);
-            }
-
-            {
-                auto sep = Gtk::manage(new Gtk::SeparatorMenuItem);
-                sep->show();
-                context_menu->append(*sep);
-            }
-
-            for (const auto &it : action_catalog) {
-                if (it.first.first == ActionID::TOOL) {
-                    auto r = core.r->tool_can_begin(it.first.second, sel_for_menu);
-                    if (r.first && r.second) {
-                        auto la = Gtk::manage(new Gtk::MenuItem(it.second.name));
-                        ToolID tool_id = it.first.second;
-                        la->signal_activate().connect([this, tool_id] {
-                            fix_cursor_pos();
-                            tool_begin(tool_id);
-                        });
-                        la->show();
-                        context_menu->append(*la);
-                    }
-                }
-            }
-#if GTK_CHECK_VERSION(3, 22, 0)
-            context_menu->popup_at_pointer((GdkEvent *)button_event);
-#else
-            context_menu->popup(0, gtk_get_current_event_time());
-#endif
+            create_context_menu(context_menu, sel_for_menu);
+            need_menu = true;
         }
     }
+
+    if (need_menu) {
+#if GTK_CHECK_VERSION(3, 22, 0)
+        context_menu->popup_at_pointer((GdkEvent *)button_event);
+#else
+        context_menu->popup(0, gtk_get_current_event_time());
+#endif
+    }
     return false;
 }
 
@@ -1116,13 +1258,17 @@ void ImpBase::tool_process(const ToolResponse &resp)
         update_highlights();
     }
     if (!no_update) {
+        canvas->fast_draw = resp.fast_draw;
         canvas_update();
+        canvas->fast_draw = false;
         canvas->set_selection(core.r->selection);
     }
     if (resp.layer != 10000) {
         canvas->property_work_layer() = resp.layer;
     }
     if (resp.next_tool != ToolID::NONE) {
+        highlights.clear();
+        update_highlights();
         ToolArgs args;
         args.coords = canvas->get_cursor_pos();
         args.keep_selection = true;
@@ -1176,6 +1322,9 @@ void ImpBase::sc(void)
                 if (poly->usage && poly->usage->get_type() == PolygonUsage::Type::PLANE) {
                     sel_extra.emplace(poly->usage->get_uuid(), ObjectType::PLANE);
                 }
+                if (poly->usage && poly->usage->get_type() == PolygonUsage::Type::KEEPOUT) {
+                    sel_extra.emplace(poly->usage->get_uuid(), ObjectType::KEEPOUT);
+                }
             } break;
             default:;
             }
@@ -1196,6 +1345,7 @@ void ImpBase::handle_tool_change(ToolID id)
     if (id != ToolID::NONE) {
         main_window->tool_bar_set_tool_name(action_catalog.at({ActionID::TOOL, id}).name);
         main_window->tool_bar_set_tool_tip("");
+        main_window->hud_hide();
     }
     main_window->tool_bar_set_visible(id != ToolID::NONE);
 }
@@ -1207,15 +1357,71 @@ void ImpBase::handle_warning_selected(const Coordi &pos)
 
 bool ImpBase::handle_broadcast(const json &j)
 {
-    std::string op = j.at("op");
+    const std::string op = j.at("op");
     if (op == "present") {
         main_window->present();
         return true;
     }
     else if (op == "save") {
-        core.r->save();
+        if (!read_only)
+            core.r->save();
+        return true;
+    }
+    else if (op == "close") {
+        delete main_window;
+        return true;
+    }
+    else if (op == "preferences") {
+        const auto &prefs = j.at("preferences");
+        preferences.load_from_json(prefs);
         return true;
     }
     return false;
 }
+
+void ImpBase::set_monitor_files(const std::set<std::string> &files)
+{
+    for (const auto &filename : files) {
+        if (file_monitors.count(filename) == 0) {
+            auto mon = Gio::File::create_for_path(filename)->monitor_file();
+            mon->signal_changed().connect(sigc::mem_fun(this, &ImpBase::handle_file_changed));
+            file_monitors[filename] = mon;
+        }
+    }
+    map_erase_if(file_monitors, [this, files](auto &it) { return files.count(it.first) == 0; });
+}
+
+void ImpBase::set_monitor_items(const std::set<std::pair<ObjectType, UUID>> &items)
+{
+    std::set<std::string> filenames;
+    std::transform(items.begin(), items.end(), std::inserter(filenames, filenames.begin()),
+                   [this](const auto &it) { return pool->get_filename(it.first, it.second); });
+    set_monitor_files(filenames);
+}
+
+void ImpBase::handle_file_changed(const Glib::RefPtr<Gio::File> &file1, const Glib::RefPtr<Gio::File> &file2,
+                                  Gio::FileMonitorEvent ev)
+{
+    main_window->show_nonmodal("Pool has changed", "Reload pool", [this] { trigger_action(ActionID::RELOAD_POOL); },
+                               "This will clear the undo/redo history");
+}
+
+void ImpBase::set_read_only(bool v)
+{
+    read_only = v;
+}
+
+void ImpBase::tool_update_data(std::unique_ptr<ToolData> &data)
+{
+
+    if (core.r->tool_is_active()) {
+        ToolArgs args;
+        args.type = ToolEventType::DATA;
+        args.data = std::move(data);
+        args.coords = canvas->get_cursor_pos();
+        ToolResponse r = core.r->tool_update(args);
+        tool_process(r);
+    }
+}
+
 } // namespace horizon
