@@ -15,11 +15,19 @@ protected:
     void construct() override;
     bool handle_broadcast(const json &j) override;
     void handle_maybe_drag() override;
+    void update_action_sensitivity() override;
+    void apply_preferences() override;
 
     ActionCatalogItem::Availability get_editor_type_for_action() const
     {
         return ActionCatalogItem::AVAILABLE_IN_BOARD;
     };
+    ObjectType get_editor_type() const override
+    {
+        return ObjectType::BOARD;
+    }
+
+    std::string get_hud_text(std::set<SelectableRef> &sel) override;
 
 private:
     void canvas_update() override;
@@ -30,11 +38,18 @@ private:
     class FabOutputWindow *fab_output_window = nullptr;
     class View3DWindow *view_3d_window = nullptr;
     class StepExportWindow *step_export_window = nullptr;
+    class TuningWindow *tuning_window = nullptr;
     bool cross_probing_enabled = false;
 
     Coordf cursor_pos_drag_begin;
     Target target_drag_begin;
 
     void handle_drag();
+    void handle_measure_tracks(const ActionConnection &a);
+
+    class CanvasAnnotation *text_owner_annotation = nullptr;
+    std::map<UUID, UUID> text_owners;
+    void update_text_owners();
+    void update_text_owner_annotation();
 };
 } // namespace horizon
