@@ -8,11 +8,14 @@
 #include "pool/part.hpp"
 #include "util/gtk_util.hpp"
 #include "util/util.hpp"
+#include "util/pool_completion.hpp"
 #include "util/changeable.hpp"
+#include "util/str_util.hpp"
 #include "widgets/chooser_buttons.hpp"
 #include "widgets/pool_browser.hpp"
 #include "widgets/spin_button_dim.hpp"
 #include "widgets/parameter_set_editor.hpp"
+#include "hud_util.hpp"
 
 namespace horizon {
 ImpPackage::ImpPackage(const std::string &package_filename, const std::string &pool_path)
@@ -26,6 +29,14 @@ ImpPackage::ImpPackage(const std::string &package_filename, const std::string &p
 void ImpPackage::canvas_update()
 {
     canvas->update(*core_package.get_canvas_data());
+    warnings_box->update(core_package.get_package()->warnings);
+}
+
+void ImpPackage::apply_preferences()
+{
+    if (view_3d_window)
+        view_3d_window->set_smooth_zoom(preferences.zoom.smooth_zoom_3d);
+    ImpLayer::apply_preferences();
 }
 
 class ModelEditor : public Gtk::Box, public Changeable {
@@ -201,7 +212,6 @@ ModelEditor::ModelEditor(ImpPackage *iimp, const UUID &iuu) : Gtk::Box(Gtk::ORIE
                 entry->set_text(mfn);
                 imp->view_3d_window->update(true);
             }
-
         });
         box->pack_end(*browse_button, false, false, 0);
 
@@ -333,6 +343,35 @@ ModelEditor::ModelEditor(ImpPackage *iimp, const UUID &iuu) : Gtk::Box(Gtk::ORIE
     update_all();
 }
 
+std::string ImpPackage::get_hud_text(std::set<SelectableRef> &sel)
+{
+    std::string s;
+    if (sel_count_type(sel, ObjectType::PAD) == 1) {
+        const auto &pad = core_package.get_package()->pads.at(sel_find_one(sel, ObjectType::PAD));
+        s += "<b>Pad " + pad.name + "</b>\n";
+        for (const auto &it : pad.parameter_set) {
+            s += parameter_id_to_name(it.first) + ": " + dim_to_string(it.second, true) + "\n";
+        }
+        sel_erase_type(sel, ObjectType::PAD);
+    }
+    else if (sel_count_type(sel, ObjectType::PAD) == 2) {
+        const auto &pkg = core_package.get_package();
+        std::vector<const Pad *> pads;
+        for (const auto &it : sel) {
+            if (it.type == ObjectType::PAD) {
+                pads.push_back(&pkg->pads.at(it.uuid));
+            }
+        }
+        assert(pads.size() == 2);
+        s += "<b>2 Pads</b>\n";
+        s += "ΔX:" + dim_to_string(std::abs(pads.at(0)->placement.shift.x - pads.at(1)->placement.shift.x)) + "\n";
+        s += "ΔY:" + dim_to_string(std::abs(pads.at(0)->placement.shift.y - pads.at(1)->placement.shift.y));
+        sel_erase_type(sel, ObjectType::PAD);
+    }
+    trim(s);
+    return s;
+}
+
 void ModelEditor::update_all()
 {
     if (imp->core_package.models.count(imp->current_model) == 0) {
@@ -354,7 +393,9 @@ std::string ImpPackage::ask_3d_model_filename(const std::string &current_filenam
     auto filter = Gtk::FileFilter::create();
     filter->set_name("STEP models");
     filter->add_pattern("*.step");
+    filter->add_pattern("*.STEP");
     filter->add_pattern("*.stp");
+    filter->add_pattern("*.STP");
     chooser->add_filter(filter);
     if (current_filename.size()) {
         chooser->set_filename(Glib::build_filename(pool->get_base_path(), current_filename));
@@ -386,6 +427,27 @@ std::string ImpPackage::ask_3d_model_filename(const std::string &current_filenam
     return "";
 }
 
+void ImpPackage::update_action_sensitivity()
+{
+    auto sel = canvas->get_selection();
+    set_action_sensitive(make_action(ActionID::EDIT_PADSTACK),
+                         sockets_connected && std::any_of(sel.begin(), sel.end(), [](const auto &x) {
+                             return x.type == ObjectType::PAD;
+                         }));
+
+    ImpBase::update_action_sensitivity();
+}
+
+void ImpPackage::update_monitor()
+{
+    std::set<std::pair<ObjectType, UUID>> items;
+    const auto pkg = core_package.get_package();
+    for (const auto &it : pkg->pads) {
+        items.emplace(ObjectType::PADSTACK, it.second.pool_padstack->uuid);
+    }
+    set_monitor_items(items);
+}
+
 void ImpPackage::construct()
 {
     ImpLayer::construct_layer_box();
@@ -460,6 +522,7 @@ void ImpPackage::construct()
                 current_model = uu;
                 view_3d_window->update();
                 ed->update_all();
+                core_package.set_needs_save();
             }
         });
         box->pack_end(*button_add, false, false, 0);
@@ -505,6 +568,13 @@ void ImpPackage::construct()
         view_3d_window->present();
     });
 
+    connect_action(ActionID::EDIT_PADSTACK, [this](const auto &a) {
+        auto sel = canvas->get_selection();
+        if (auto uu = sel_find_one(sel, ObjectType::PAD)) {
+            this->edit_pool_item(ObjectType::PADSTACK, core_package.get_package()->pads.at(uu).pool_padstack->uuid);
+        }
+    });
+
     footprint_generator_window = FootprintGeneratorWindow::create(main_window, &core_package);
     footprint_generator_window->signal_generated().connect(sigc::mem_fun(this, &ImpBase::canvas_update_from_pp));
 
@@ -517,29 +587,7 @@ void ImpPackage::construct()
         button->show();
         button->signal_clicked().connect([this, parameter_window] { parameter_window->present(); });
     }
-    {
-        auto button = Gtk::manage(new Gtk::Button("Polygon expand"));
-        parameter_window->add_button(button);
-        button->signal_clicked().connect([this, parameter_window] {
-            auto sel = canvas->get_selection();
-            if (sel.size() == 1) {
-                auto &s = *sel.begin();
-                if (s.type == ObjectType::POLYGON_EDGE || s.type == ObjectType::POLYGON_VERTEX) {
-                    auto poly = core.r->get_polygon(s.uuid);
-                    if (!poly->has_arcs()) {
-                        std::stringstream ss;
-                        ss.imbue(std::locale("C"));
-                        ss << "expand-polygon [ " << poly->parameter_class << " ";
-                        for (const auto &it : poly->vertices) {
-                            ss << it.position.x << " " << it.position.y << " ";
-                        }
-                        ss << "]\n";
-                        parameter_window->insert_text(ss.str());
-                    }
-                }
-            }
-        });
-    }
+    parameter_window_add_polygon_expand(parameter_window);
     {
         auto button = Gtk::manage(new Gtk::Button("Insert courtyard program"));
         parameter_window->add_button(button);
@@ -613,6 +661,8 @@ void ImpPackage::construct()
 
     auto entry_name = header_button->add_entry("Name");
     auto entry_manufacturer = header_button->add_entry("Manufacturer");
+    entry_manufacturer->set_completion(create_pool_manufacturer_completion(pool.get()));
+
     auto entry_tags = header_button->add_entry("Tags");
 
     auto browser_alt_button = Gtk::manage(new PoolBrowserButton(ObjectType::PACKAGE, pool.get()));
@@ -641,9 +691,11 @@ void ImpPackage::construct()
 
     auto hamburger_menu = add_hamburger_menu();
     hamburger_menu->append("Import DXF", "win.import_dxf");
-
     add_tool_action(ToolID::IMPORT_DXF, "import_dxf");
 
+    hamburger_menu->append("Reload pool", "win.reload_pool");
+    main_window->add_action("reload_pool", [this] { trigger_action(ActionID::RELOAD_POOL); });
+
     {
         auto button = Gtk::manage(new Gtk::Button("Footprint gen."));
         button->signal_clicked().connect([this] {
@@ -655,6 +707,8 @@ void ImpPackage::construct()
         main_window->header->pack_end(*button);
     }
 
+    update_monitor();
+
     core_package.signal_save().connect(
             [this, entry_name, entry_manufacturer, entry_tags, header_button, browser_alt_button] {
                 auto pkg = core_package.get_package(false);
