@@ -12,11 +12,20 @@ public:
 
 protected:
     void construct() override;
+    void apply_preferences() override;
 
     ActionCatalogItem::Availability get_editor_type_for_action() const
     {
         return ActionCatalogItem::AVAILABLE_IN_PACKAGE;
     };
+    ObjectType get_editor_type() const override
+    {
+        return ObjectType::PACKAGE;
+    }
+
+    std::string get_hud_text(std::set<SelectableRef> &sel) override;
+    void update_action_sensitivity() override;
+    void update_monitor() override;
 
 private:
     void canvas_update() override;
