@@ -13,6 +13,10 @@ protected:
     {
         return ActionCatalogItem::AVAILABLE_IN_PADSTACK;
     };
+    ObjectType get_editor_type() const override
+    {
+        return ObjectType::PADSTACK;
+    }
 
 private:
     void canvas_update() override;
