@@ -5,6 +5,10 @@
 #include "rules/rules_window.hpp"
 #include "widgets/sheet_box.hpp"
 #include "util/gtk_util.hpp"
+#include "util/util.hpp"
+#include "util/str_util.hpp"
+#include "hud_util.hpp"
+#include "bom_export_window.hpp"
 
 namespace horizon {
 ImpSchematic::ImpSchematic(const std::string &schematic_filename, const std::string &block_filename,
@@ -115,6 +119,11 @@ bool ImpSchematic::handle_broadcast(const json &j)
             part_from_project_manager = j.at("part").get<std::string>();
             tool_begin(ToolID::ADD_PART);
         }
+        if (op == "assign-part") {
+            main_window->present();
+            part_from_project_manager = j.at("part").get<std::string>();
+            tool_begin(ToolID::ASSIGN_PART);
+        }
         else if (op == "highlight" && cross_probing_enabled) {
             if (!core_schematic.tool_is_active()) {
                 highlights.clear();
@@ -187,6 +196,13 @@ void ImpSchematic::handle_selection_cross_probe()
     send_json(j);
 }
 
+int ImpSchematic::get_board_pid()
+{
+    json j;
+    j["op"] = "get-board-pid";
+    return this->send_json(j);
+}
+
 void ImpSchematic::construct()
 {
 
@@ -219,6 +235,9 @@ void ImpSchematic::construct()
     hamburger_menu->append("Schematic properties", "win.sch_properties");
     add_tool_action(ToolID::EDIT_SCHEMATIC_PROPERTIES, "sch_properties");
 
+    hamburger_menu->append("BOM Export", "win.bom_export");
+    main_window->add_action("bom_export", [this] { trigger_action(ActionID::BOM_EXPORT_WINDOW); });
+
     hamburger_menu->append("Export PDF", "win.export_pdf");
 
     main_window->add_action("export_pdf", [this] { handle_export_pdf(); });
@@ -250,12 +269,13 @@ void ImpSchematic::construct()
 
     connect_action(ActionID::PLACE_PART, [this](const auto &conn) {
         if (!sockets_connected) {
-            tool_begin(ToolID::ADD_PART);
+            this->tool_begin(ToolID::ADD_PART);
         }
         else {
             json j;
             j["op"] = "show-browser";
-            send_json(j);
+            allow_set_foreground_window(mgr_pid);
+            this->send_json(j);
         }
     });
 
@@ -274,17 +294,93 @@ void ImpSchematic::construct()
                     j["selection"].push_back(k);
                 }
             }
-            send_json(j);
+            allow_set_foreground_window(this->get_board_pid());
+            this->send_json(j);
         });
         set_action_sensitive(make_action(ActionID::TO_BOARD), false);
         auto button = create_action_button(make_action(ActionID::TO_BOARD));
         button->set_tooltip_text("Place selected components on board");
         button->show();
         main_window->header->pack_end(*button);
+
+        connect_action(ActionID::SHOW_IN_BROWSER, [this](const auto &conn) {
+            for (const auto &it : canvas->get_selection()) {
+                auto sheet = core_schematic.get_sheet();
+                if (it.type == ObjectType::SCHEMATIC_SYMBOL) {
+                    auto &sym = sheet->symbols.at(it.uuid);
+                    if (sym.component->part) {
+                        json j;
+                        j["op"] = "show-in-browser";
+                        j["part"] = (std::string)sym.component->part->uuid;
+                        this->send_json(j);
+                        break;
+                    }
+                }
+            }
+        });
+
+        connect_action(ActionID::SAVE_RELOAD_NETLIST, [this](const auto &conn) {
+            this->trigger_action(ActionID::SAVE);
+            json j;
+            j["op"] = "reload-netlist";
+            this->send_json(j);
+        });
+        set_action_sensitive(make_action(ActionID::SAVE_RELOAD_NETLIST), false);
     }
 
     connect_action(ActionID::MOVE_TO_OTHER_SHEET,
                    std::bind(&ImpSchematic::handle_move_to_other_sheet, this, std::placeholders::_1));
+
+
+    connect_action(ActionID::HIGHLIGHT_NET, [this](const auto &a) {
+        highlights.clear();
+        auto sheet = core_schematic.get_sheet();
+        for (const auto &it : canvas->get_selection()) {
+            switch (it.type) {
+            case ObjectType::LINE_NET: {
+                auto &li = sheet->net_lines.at(it.uuid);
+                if (li.net) {
+                    highlights.emplace(ObjectType::NET, li.net->uuid);
+                }
+            } break;
+            case ObjectType::NET_LABEL: {
+                auto &la = sheet->net_labels.at(it.uuid);
+                if (la.junction->net) {
+                    highlights.emplace(ObjectType::NET, la.junction->net->uuid);
+                }
+            } break;
+            case ObjectType::POWER_SYMBOL: {
+                auto &sym = sheet->power_symbols.at(it.uuid);
+                if (sym.junction->net) {
+                    highlights.emplace(ObjectType::NET, sym.junction->net->uuid);
+                }
+            } break;
+            case ObjectType::JUNCTION: {
+                auto &ju = sheet->junctions.at(it.uuid);
+                if (ju.net) {
+                    highlights.emplace(ObjectType::NET, ju.net->uuid);
+                }
+            } break;
+            default:;
+            }
+        }
+        this->update_highlights();
+    });
+
+    bom_export_window =
+            BOMExportWindow::create(main_window, core_schematic.get_block(), core_schematic.get_bom_export_settings());
+
+    connect_action(ActionID::BOM_EXPORT_WINDOW, [this](const auto &c) { bom_export_window->present(); });
+    connect_action(ActionID::EXPORT_BOM, [this](const auto &c) {
+        bom_export_window->present();
+        bom_export_window->generate();
+    });
+
+    bom_export_window->signal_changed().connect([this] { core_schematic.set_needs_save(); });
+    core.r->signal_tool_changed().connect([this](ToolID t) { bom_export_window->set_can_export(t == ToolID::NONE); });
+    core.r->signal_rebuilt().connect([this] { bom_export_window->update_preview(); });
+
+
     grid_spin_button->set_sensitive(false);
 
     rules_window->signal_goto().connect([this](Coordi location, UUID sheet) {
@@ -310,45 +406,101 @@ void ImpSchematic::construct()
 
 void ImpSchematic::update_action_sensitivity()
 {
+    auto sel = canvas->get_selection();
     if (sockets_connected) {
         json req;
         req["op"] = "has-board";
         bool has_board = send_json(req);
+        set_action_sensitive(make_action(ActionID::SAVE_RELOAD_NETLIST), has_board);
         if (!has_board) {
+
             set_action_sensitive(make_action(ActionID::TO_BOARD), false);
         }
         else {
-            auto sel = canvas->get_selection();
+
             auto has_sym = std::any_of(sel.begin(), sel.end(),
                                        [](const auto &x) { return x.type == ObjectType::SCHEMATIC_SYMBOL; });
             set_action_sensitive(make_action(ActionID::TO_BOARD), has_sym);
+            set_action_sensitive(make_action(ActionID::SHOW_IN_BROWSER), has_sym);
         }
     }
     else {
         set_action_sensitive(make_action(ActionID::TO_BOARD), false);
+        set_action_sensitive(make_action(ActionID::SHOW_IN_BROWSER), false);
     }
-    set_action_sensitive(make_action(ActionID::MOVE_TO_OTHER_SHEET), canvas->get_selection().size() > 0);
+    set_action_sensitive(make_action(ActionID::MOVE_TO_OTHER_SHEET), sel.size() > 0);
+
+    set_action_sensitive(make_action(ActionID::HIGHLIGHT_NET), std::any_of(sel.begin(), sel.end(), [](const auto &x) {
+                             switch (x.type) {
+                             case ObjectType::LINE_NET:
+                             case ObjectType::NET_LABEL:
+                             case ObjectType::JUNCTION:
+                             case ObjectType::POWER_SYMBOL:
+                                 return true;
+
+                             default:
+                                 return false;
+                             }
+                         }));
 
     ImpBase::update_action_sensitivity();
 }
 
+std::string ImpSchematic::get_hud_text(std::set<SelectableRef> &sel)
+{
+    std::string s;
+    if (sel_count_type(sel, ObjectType::SCHEMATIC_SYMBOL) == 1) {
+        const auto &sym = core_schematic.get_sheet()->symbols.at(sel_find_one(sel, ObjectType::SCHEMATIC_SYMBOL));
+        s += "<b>Symbol " + sym.component->refdes + "</b>\n";
+        s += get_hud_text_for_part(sym.component->part);
+        sel_erase_type(sel, ObjectType::SCHEMATIC_SYMBOL);
+    }
+    trim(s);
+    return s;
+}
+
 void ImpSchematic::handle_export_pdf()
 {
-    GtkFileChooserNative *native = gtk_file_chooser_native_new("Save PDF", GTK_WINDOW(main_window->gobj()),
-                                                               GTK_FILE_CHOOSER_ACTION_SAVE, "_Open", "_Cancel");
-    auto chooser = Glib::wrap(GTK_FILE_CHOOSER(native));
-    chooser->set_do_overwrite_confirmation(true);
-    if (last_pdf_filename.size()) {
-        chooser->set_filename(last_pdf_filename);
-    }
-    else {
-        chooser->set_current_name("schematic.pdf");
-    }
+    std::string filename;
+    {
+        GtkFileChooserNative *native = gtk_file_chooser_native_new("Save PDF", GTK_WINDOW(main_window->gobj()),
+                                                                   GTK_FILE_CHOOSER_ACTION_SAVE, "_Save", "_Cancel");
+        auto chooser = Glib::wrap(GTK_FILE_CHOOSER(native));
+        chooser->set_do_overwrite_confirmation(true);
+        if (last_pdf_filename.size()) {
+            chooser->set_filename(last_pdf_filename);
+        }
+        else {
+            chooser->set_current_name("schematic.pdf");
+        }
 
-    if (gtk_native_dialog_run(GTK_NATIVE_DIALOG(native)) == GTK_RESPONSE_ACCEPT) {
-        std::string fn = chooser->get_filename();
-        last_pdf_filename = fn;
-        export_pdf(fn, *core.c->get_schematic(), core.r);
+        if (gtk_native_dialog_run(GTK_NATIVE_DIALOG(native)) == GTK_RESPONSE_ACCEPT) {
+            filename = chooser->get_filename();
+            last_pdf_filename = filename;
+        }
+    }
+    if (filename.size()) {
+        while (1) {
+            std::string error_str;
+            try {
+                export_pdf(filename, *core.c->get_schematic(), core.r);
+                break;
+            }
+            catch (const std::exception &e) {
+                error_str = e.what();
+            }
+            catch (...) {
+                error_str = "unknown error";
+            }
+            if (error_str.size()) {
+                Gtk::MessageDialog dia(*main_window, "PDF export error", false, Gtk::MESSAGE_ERROR, Gtk::BUTTONS_NONE);
+                dia.set_secondary_text(error_str);
+                dia.add_button("Cancel", Gtk::RESPONSE_CANCEL);
+                dia.add_button("Retry", 1);
+                if (dia.run() != 1)
+                    break;
+            }
+        }
     }
 }
 
@@ -421,6 +573,8 @@ void ImpSchematic::handle_drag()
 
     if (start) {
         {
+            highlights.clear();
+            update_highlights();
             ToolArgs args;
             args.coords = target_drag_begin.p;
             ToolResponse r = core.r->tool_begin(ToolID::DRAW_NET, args, imp_interface.get());
