@@ -19,6 +19,12 @@ protected:
     {
         return ActionCatalogItem::AVAILABLE_IN_SCHEMATIC;
     };
+    ObjectType get_editor_type() const override
+    {
+        return ObjectType::SCHEMATIC;
+    }
+
+    std::string get_hud_text(std::set<SelectableRef> &sel) override;
 
 private:
     void canvas_update() override;
@@ -42,6 +48,10 @@ private:
     Coordf cursor_pos_drag_begin;
     Target target_drag_begin;
 
+    class BOMExportWindow *bom_export_window;
+
     void handle_drag();
+
+    int get_board_pid();
 };
 } // namespace horizon
