@@ -21,11 +21,19 @@ public:
     Gtk::HeaderBar *header = nullptr;
     Glib::RefPtr<Gtk::Builder> builder;
 
+    Gtk::Button *pool_reload_button = nullptr;
+
     void tool_bar_set_visible(bool v);
     void tool_bar_set_tool_name(const std::string &s);
     void tool_bar_set_tool_tip(const std::string &s);
     void tool_bar_flash(const std::string &s);
 
+    void hud_update(const std::string &s);
+    void hud_hide();
+
+    void show_nonmodal(const std::string &la, const std::string &button, std::function<void(void)> fn,
+                       const std::string &la2 = "");
+
     // virtual ~MainWindow();
 private:
     Gtk::Box *gl_container = nullptr;
@@ -37,6 +45,19 @@ private:
     sigc::connection tip_timeout_connection;
     bool tool_bar_queue_close = false;
 
+    Gtk::Revealer *hud = nullptr;
+    Gtk::Label *hud_label = nullptr;
+
+    sigc::connection hud_timeout_connection;
+    bool hud_queue_close = false;
+
+    Gtk::Button *nonmodal_close_button = nullptr;
+    Gtk::Button *nonmodal_button = nullptr;
+    Gtk::Revealer *nonmodal_rev = nullptr;
+    Gtk::Label *nonmodal_label = nullptr;
+    Gtk::Label *nonmodal_label2 = nullptr;
+    std::function<void(void)> nonmodal_fn;
+
     void sc(void);
     void cm(const horizon::Coordi &cursor_pos);
 };
