@@ -22,14 +22,13 @@ public:
     Pad(const UUID &uu, const json &, Pool &pool);
     Pad(const UUID &uu, const Padstack *ps);
     UUID uuid;
-    const Padstack *pool_padstack;
+    uuid_ptr<const Padstack> pool_padstack;
     Padstack padstack;
     Placement placement;
     std::string name;
     ParameterSet parameter_set;
 
     uuid_ptr<Net> net = nullptr;
-    UUID net_segment;
 
     virtual UUID get_uuid() const;
     json serialize() const;
