@@ -4,7 +4,7 @@
 #include "board/via_padstack_provider.hpp"
 #include "canvas/canvas_gl.hpp"
 #include "clipper/clipper.hpp"
-#include "geometry/shape_convex.h"
+#include "geometry/shape_simple.h"
 #include "router/pns_debug_decorator.h"
 #include "router/pns_solid.h"
 #include "router/pns_topology.h"
@@ -71,11 +71,6 @@ public:
 
     virtual int Clearance(const PNS::ITEM *aA, const PNS::ITEM *aB) const override;
     virtual int Clearance(int aNetCode) const override;
-    virtual void OverrideClearance(bool aEnable, int aNetA = 0, int aNetB = 0, int aClearance = 0) override;
-    virtual void UseDpGap(bool aUseDpGap) override
-    {
-        m_useDpGap = aUseDpGap;
-    }
     virtual int DpCoupledNet(int aNet) override;
     virtual int DpNetPolarity(int aNet) override;
     virtual bool DpNetPair(PNS::ITEM *aItem, int &aNetP, int &aNetN) override;
@@ -85,6 +80,7 @@ private:
     const horizon::Board *m_board;
     horizon::BoardRules *m_rules;
     PNS_HORIZON_IFACE *m_iface = nullptr;
+    std::vector<horizon::RuleClearanceCopperKeepout *> m_rules_keepout;
 
     bool m_useDpGap = false;
 };
@@ -98,6 +94,8 @@ PNS_HORIZON_RULE_RESOLVER::PNS_HORIZON_RULE_RESOLVER(const horizon::Board *aBoar
     PNS::TOPOLOGY topo(world);
 
     m_iface = static_cast<PNS_HORIZON_IFACE *>(m_router->GetInterface());
+    m_rules_keepout = horizon::dynamic_cast_vector<horizon::RuleClearanceCopperKeepout *>(
+            m_rules->get_rules_sorted(horizon::RuleID::CLEARANCE_COPPER_KEEPOUT));
 }
 
 PNS_HORIZON_RULE_RESOLVER::~PNS_HORIZON_RULE_RESOLVER()
@@ -189,6 +187,22 @@ int PNS_HORIZON_RULE_RESOLVER::Clearance(const PNS::ITEM *aA, const PNS::ITEM *a
         return clearance->get_clearance(pt, horizon::PatchType::HOLE_NPTH) + clearance->routing_offset;
     }
 
+    if ((parent_a && parent_a->keepout) || (parent_b && parent_b->keepout)) { // one is keepout
+        bool a_is_keepout = parent_a && parent_a->keepout;
+        auto net = net_a ? net_a : net_b; // only one has net
+        horizon::KeepoutContour keepout_contour;
+        keepout_contour.keepout = a_is_keepout ? parent_a->keepout : parent_b->keepout;
+        ;
+        keepout_contour.pkg = a_is_keepout ? parent_a->package : parent_b->package;
+        for (const auto &rule : m_rules_keepout) {
+            if (rule->enabled && rule->match.match(net) && rule->match_keepout.match(&keepout_contour)) {
+                auto cl = rule->get_clearance(a_is_keepout ? pt_b : pt_a);
+                return rule->routing_offset + cl;
+            }
+        }
+        return 0;
+    }
+
     int layer = UNDEFINED_LAYER;
     if (!layers_a.IsMultilayer() && !layers_b.IsMultilayer()) // all on single layer
         layer = layers_b.Start();
@@ -202,7 +216,12 @@ int PNS_HORIZON_RULE_RESOLVER::Clearance(const PNS::ITEM *aA, const PNS::ITEM *a
     assert(layer != UNDEFINED_LAYER);
 
     auto clearance = m_rules->get_clearance_copper(net_a, net_b, PNS_HORIZON_IFACE::layer_from_router(layer));
-    return clearance->get_clearance(pt_a, pt_b) + clearance->routing_offset;
+
+    int64_t routing_offset = clearance->routing_offset;
+    if (m_iface->get_override_routing_offset() >= 0)
+        routing_offset = m_iface->get_override_routing_offset();
+
+    return clearance->get_clearance(pt_a, pt_b) + routing_offset;
 }
 
 int PNS_HORIZON_RULE_RESOLVER::Clearance(int aNetCode) const
@@ -211,16 +230,6 @@ int PNS_HORIZON_RULE_RESOLVER::Clearance(int aNetCode) const
     return .1e6;
 }
 
-// fixme: ugly hack to make the optimizer respect gap width for currently routed
-// differential pair.
-void PNS_HORIZON_RULE_RESOLVER::OverrideClearance(bool aEnable, int aNetA, int aNetB, int aClearance)
-{
-    /*m_overrideEnabled = aEnable;
-    m_overrideNetA = aNetA;
-    m_overrideNetB = aNetB;
-    m_overrideClearance = aClearance;*/
-}
-
 int PNS_HORIZON_RULE_RESOLVER::DpCoupledNet(int aNet)
 {
     auto net = m_iface->get_net_for_code(aNet);
@@ -331,7 +340,7 @@ public:
             auto pt = aLine.CPoint(i);
             pts.emplace_back(pt.x, pt.y);
         }
-        lines.insert(m_canvas->add_line(pts, aWidth, horizon::ColorP::YELLOW, 10000));
+        lines.insert(m_canvas->add_line(pts, aWidth, horizon::ColorP::AIRWIRE_ROUTER, 10000));
     }
 
     void Clear() override
@@ -396,28 +405,40 @@ horizon::Net *PNS_HORIZON_IFACE::get_net_for_code(int code)
 
 const PNS_HORIZON_PARENT_ITEM *PNS_HORIZON_IFACE::get_parent(const horizon::Track *track)
 {
-    PNS_HORIZON_PARENT_ITEM it(track);
-    return &*parents.insert(it).first;
+    return get_or_create_parent(PNS_HORIZON_PARENT_ITEM(track));
 }
 
 const PNS_HORIZON_PARENT_ITEM *PNS_HORIZON_IFACE::get_parent(const horizon::BoardHole *hole)
 {
-    PNS_HORIZON_PARENT_ITEM it(hole);
-    return &*parents.insert(it).first;
+    return get_or_create_parent(PNS_HORIZON_PARENT_ITEM(hole));
 }
 
 const PNS_HORIZON_PARENT_ITEM *PNS_HORIZON_IFACE::get_parent(const horizon::Via *via)
 {
-    PNS_HORIZON_PARENT_ITEM it(via);
-    return &*parents.insert(it).first;
+    return get_or_create_parent(PNS_HORIZON_PARENT_ITEM(via));
 }
 
 const PNS_HORIZON_PARENT_ITEM *PNS_HORIZON_IFACE::get_parent(const horizon::BoardPackage *pkg, const horizon::Pad *pad)
 {
-    PNS_HORIZON_PARENT_ITEM it(pkg, pad);
-    return &*parents.insert(it).first;
+    return get_or_create_parent(PNS_HORIZON_PARENT_ITEM(pkg, pad));
 }
 
+const PNS_HORIZON_PARENT_ITEM *PNS_HORIZON_IFACE::get_parent(const horizon::Keepout *k,
+                                                             const horizon::BoardPackage *pkg)
+{
+    return get_or_create_parent(PNS_HORIZON_PARENT_ITEM(k, pkg));
+}
+
+const PNS_HORIZON_PARENT_ITEM *PNS_HORIZON_IFACE::get_or_create_parent(const PNS_HORIZON_PARENT_ITEM &it)
+{
+    auto r = std::find(parents.begin(), parents.end(), it);
+    if (r != parents.end())
+        return &(*r);
+    parents.emplace_back(it);
+    return &parents.back();
+}
+
+
 std::unique_ptr<PNS::SEGMENT> PNS_HORIZON_IFACE::syncTrack(const horizon::Track *track)
 {
     auto from = track->from.get_position();
@@ -460,6 +481,42 @@ void PNS_HORIZON_IFACE::syncOutline(const horizon::Polygon *ipoly, PNS::NODE *aW
     }
 }
 
+void PNS_HORIZON_IFACE::syncKeepout(const horizon::KeepoutContour *keepout_contour, PNS::NODE *aWorld)
+{
+    auto keepout = keepout_contour->keepout;
+    if (!horizon::BoardLayers::is_copper(keepout->polygon->layer))
+        return;
+    if (!(keepout->patch_types_cu.count(horizon::PatchType::TRACK)))
+        return;
+
+    std::set<int> layers;
+    if (keepout->all_cu_layers) {
+        for (unsigned int j = 0; j < board->get_n_inner_layers(); j++) {
+            layers.insert(-j - 1);
+        }
+        layers.insert(horizon::BoardLayers::TOP_COPPER);
+        layers.insert(horizon::BoardLayers::BOTTOM_COPPER);
+    }
+    else {
+        layers.insert(keepout->polygon->layer);
+    }
+    const auto &contour = keepout_contour->contour;
+    for (auto layer : layers) {
+        for (size_t i = 0; i < contour.size(); i++) {
+            auto from = contour[i];
+            auto to = contour[(i + 1) % contour.size()];
+
+            std::unique_ptr<PNS::SEGMENT> segment(
+                    new PNS::SEGMENT(SEG(from.X, from.Y, to.X, to.Y), PNS::ITEM::UnusedNet));
+            segment->SetWidth(10); // very small
+            segment->SetLayers(layer_to_router(layer));
+            segment->SetParent(get_parent(keepout_contour->keepout, keepout_contour->pkg));
+            segment->Mark(PNS::MK_LOCKED);
+            aWorld->Add(std::move(segment));
+        }
+    }
+}
+
 std::unique_ptr<PNS::SOLID> PNS_HORIZON_IFACE::syncPad(const horizon::BoardPackage *pkg, const horizon::Pad *pad)
 {
     horizon::Placement tr(pkg->placement);
@@ -556,7 +613,7 @@ std::unique_ptr<PNS::SOLID> PNS_HORIZON_IFACE::syncPadstack(const horizon::Padst
     solid->SetOffset(VECTOR2I(0, 0));
     solid->SetPos(VECTOR2I(tr.shift.x, tr.shift.y));
 
-    SHAPE_CONVEX *shape = new SHAPE_CONVEX();
+    SHAPE_SIMPLE *shape = new SHAPE_SIMPLE();
 
     for (auto &pt : poly_union.front()) {
         shape->Append(pt.X, pt.Y);
@@ -631,6 +688,13 @@ void PNS_HORIZON_IFACE::SyncWorld(PNS::NODE *aWorld)
             syncOutline(&it.second, aWorld);
         }
     }
+
+    auto keepout_contours = board->get_keepout_contours();
+    for (const auto &it : keepout_contours) {
+        if (horizon::BoardLayers::is_copper(it.keepout->polygon->layer))
+            syncKeepout(&it, aWorld);
+    }
+
     int worstClearance = rules->get_max_clearance();
 
     delete m_ruleResolver;
@@ -700,7 +764,6 @@ void PNS_HORIZON_IFACE::DisplayItem(const PNS::ITEM *aItem, int aColor, int aCle
 
 void PNS_HORIZON_IFACE::HideItem(PNS::ITEM *aItem)
 {
-    std::cout << "iface hide item" << std::endl;
     auto parent = aItem->Parent();
     if (parent) {
         if (parent->track) {
@@ -796,7 +859,6 @@ void PNS_HORIZON_IFACE::AddItem(PNS::ITEM *aItem)
                     conn.connect(ju);
                 }
             }
-
         };
         connect(track->from, from);
         connect(track->to, to);
