@@ -18,7 +18,9 @@ enum class RuleID {
     CLEARANCE_COPPER_OTHER,
     PLANE,
     DIFFPAIR,
-    PACKAGE_CHECKS
+    PACKAGE_CHECKS,
+    PREFLIGHT_CHECKS,
+    CLEARANCE_COPPER_KEEPOUT
 };
 
 class Rule {
