@@ -23,11 +23,12 @@ public:
     BusRipper(const UUID &uu, const json &j, class Sheet &sheet, class Block &block);
     BusRipper(const UUID &uu);
     virtual UUID get_uuid() const;
-    const UUID uuid;
+    UUID uuid;
     bool temp = false;
 
     uuid_ptr<Junction> junction;
-    bool mirror = false;
+    Orientation orientation = Orientation::UP;
+    void mirror();
     uuid_ptr<Bus> bus = nullptr;
     uuid_ptr<Bus::Member> bus_member = nullptr;
     unsigned int connection_count = 0;
