@@ -28,6 +28,7 @@ Schematic::Annotation::Annotation(const json &j)
         mode = annotation_mode_lut.lookup(j.at("mode"));
         fill_gaps = j.at("fill_gaps");
         keep = j.at("keep");
+        ignore_unknown = j.value("ignore_unknown", false);
     }
 }
 
@@ -38,6 +39,7 @@ json Schematic::Annotation::serialize() const
     j["mode"] = annotation_mode_lut.lookup_reverse(mode);
     j["fill_gaps"] = fill_gaps;
     j["keep"] = keep;
+    j["ignore_unknown"] = ignore_unknown;
     return j;
 }
 
@@ -54,6 +56,12 @@ Schematic::Schematic(const UUID &uu, const json &j, Block &iblock, Pool &pool)
     if (j.count("rules")) {
         rules.load_from_json(j.at("rules"));
     }
+    if (j.count("title_block_values")) {
+        const json &o = j["title_block_values"];
+        for (auto it = o.cbegin(); it != o.cend(); ++it) {
+            title_block_values[it.key()] = it.value();
+        }
+    }
     update_refs();
 }
 
@@ -209,7 +217,7 @@ void Schematic::smash_symbol(Sheet *sheet, SchematicSymbol *sym)
     if (sym->smashed)
         return;
     sym->smashed = true;
-    for (const auto &it : sym->symbol.texts) {
+    for (const auto &it : sym->pool_symbol->texts) {
         auto uu = UUID::random();
         auto &x = sheet->texts.emplace(uu, uu).first->second;
         x.from_smash = true;
@@ -224,6 +232,14 @@ void Schematic::smash_symbol(Sheet *sheet, SchematicSymbol *sym)
     }
 }
 
+static void replace_substr(std::string &str, const std::string &needle, const std::string &rep)
+{
+    auto pos = str.find(needle);
+    if (pos != std::string::npos) {
+        str.replace(pos, needle.size(), rep);
+    }
+}
+
 void Schematic::unsmash_symbol(Sheet *sheet, SchematicSymbol *sym)
 {
     assert(sheet == &sheets.at(sheet->uuid));
@@ -401,21 +417,26 @@ void Schematic::expand(bool careful)
             for (const auto &it_conn : comp->connections) {
                 const Gate &gate_conn = comp->entity->gates.at(it_conn.first.at(0));
                 const Pin &pin = gate_conn.unit->pins.at(it_conn.first.at(1));
-                if ((gate_conn.uuid == schsym.gate->uuid) && it_conn.second.net) { // connection with net
+                if ((gate_conn.uuid == schsym.gate->uuid)) { // connection with net
                     SymbolPin &sympin = schsym.symbol.pins.at(pin.uuid);
-                    sympin.connector_style = SymbolPin::ConnectorStyle::NONE;
-                    if (sympin.net_segment) {
-                        auto &ns = net_segments.at(sympin.net_segment);
-                        if (ns.is_bus()) {
-                            throw std::runtime_error("illegal bus-pin connection");
-                        }
-                        if (ns.net == nullptr) {
-                            ns.net = it_conn.second.net;
-                        }
-                        if (ns.net != it_conn.second.net) {
-                            throw std::runtime_error("illegal connection");
+                    if (it_conn.second.net) {
+                        sympin.connector_style = SymbolPin::ConnectorStyle::NONE;
+                        if (sympin.net_segment) {
+                            auto &ns = net_segments.at(sympin.net_segment);
+                            if (ns.is_bus()) {
+                                throw std::runtime_error("illegal bus-pin connection");
+                            }
+                            if (ns.net == nullptr) {
+                                ns.net = it_conn.second.net;
+                            }
+                            if (ns.net != it_conn.second.net) {
+                                throw std::runtime_error("illegal connection");
+                            }
                         }
                     }
+                    else {
+                        sympin.connector_style = SymbolPin::ConnectorStyle::NC;
+                    }
                 }
             }
         }
@@ -529,6 +550,25 @@ void Schematic::expand(bool careful)
             }
         }
     }
+    for (auto &it_sheet : sheets) {
+        Sheet &sheet = it_sheet.second;
+        if (sheet.pool_frame) {
+            std::map<std::string, std::string> values = title_block_values;
+            for (const auto &it_v : sheet.title_block_values) {
+                values[it_v.first] = it_v.second;
+            }
+            sheet.frame = *sheet.pool_frame;
+            for (auto &it_text : sheet.frame.texts) {
+                auto &txt = it_text.second.text;
+                replace_substr(txt, "$sheet_idx", std::to_string(sheet.index));
+                replace_substr(txt, "$sheet_total", std::to_string(sheets.size()));
+                replace_substr(txt, "$sheet_title", sheet.name);
+                for (const auto &it_v : values) {
+                    replace_substr(txt, "$" + it_v.first, it_v.second);
+                }
+            }
+        }
+    }
 }
 
 void Schematic::annotate()
@@ -599,7 +639,8 @@ void Schematic::annotate()
                     si = std::stoi(ss);
                 }
                 catch (const std::invalid_argument &e) {
-                    sym->component->refdes = "?";
+                    if (!annotation.ignore_unknown)
+                        sym->component->refdes = "?";
                 }
                 if (si > 0) {
                     v.push_back(si);
@@ -646,7 +687,8 @@ void Schematic::annotate()
 }
 
 Schematic::Schematic(const Schematic &sch)
-    : uuid(sch.uuid), block(sch.block), name(sch.name), sheets(sch.sheets), rules(sch.rules), annotation(sch.annotation)
+    : uuid(sch.uuid), block(sch.block), name(sch.name), sheets(sch.sheets), rules(sch.rules),
+      title_block_values(sch.title_block_values), annotation(sch.annotation)
 {
     update_refs();
 }
@@ -658,6 +700,7 @@ void Schematic::operator=(const Schematic &sch)
     name = sch.name;
     sheets = sch.sheets;
     rules = sch.rules;
+    title_block_values = sch.title_block_values;
     annotation = sch.annotation;
     update_refs();
 }
@@ -685,6 +728,17 @@ void Schematic::update_refs()
             line.net.update(block->nets);
             line.bus.update(block->buses);
         }
+        for (auto &it_line : sheet.lines) {
+            Line &line = it_line.second;
+            line.from.update(sheet.junctions);
+            line.to.update(sheet.junctions);
+        }
+        for (auto &it_arc : sheet.arcs) {
+            Arc &arc = it_arc.second;
+            arc.from.update(sheet.junctions);
+            arc.to.update(sheet.junctions);
+            arc.center.update(sheet.junctions);
+        }
         for (auto &it_junc : sheet.junctions) {
             it_junc.second.net.update(block->nets);
             it_junc.second.bus.update(block->buses);
@@ -714,7 +768,7 @@ json Schematic::serialize() const
     j["name"] = name;
     j["annotation"] = annotation.serialize();
     j["rules"] = rules.serialize();
-
+    j["title_block_values"] = title_block_values;
 
     j["sheets"] = json::object();
     for (const auto &it : sheets) {
