@@ -10,11 +10,15 @@
 namespace horizon {
 
 Sheet::Sheet(const UUID &uu, const json &j, Block &block, Pool &pool)
-    : uuid(uu), name(j.at("name").get<std::string>()), index(j.at("index").get<unsigned int>())
+    : uuid(uu), name(j.at("name").get<std::string>()), index(j.at("index").get<unsigned int>()), frame(UUID())
 {
     Logger::log_info("loading sheet " + name, Logger::Domain::SCHEMATIC);
     if (j.count("frame")) {
-        frame = Frame(j.at("frame"));
+        const auto &v = j.at("frame");
+        if (v.is_string()) {
+            pool_frame = pool.get_frame(UUID(v.get<std::string>()));
+            frame = *pool_frame;
+        }
     }
     {
         const json &o = j["junctions"];
@@ -78,9 +82,30 @@ Sheet::Sheet(const UUID &uu, const json &j, Block &block, Pool &pool)
             bus_labels.emplace(std::make_pair(u, BusLabel(u, it.value(), *this, block)));
         }
     }
+    if (j.count("lines")) {
+        const json &o = j["lines"];
+        for (auto it = o.cbegin(); it != o.cend(); ++it) {
+            auto u = UUID(it.key());
+            load_and_log(lines, ObjectType::LINE, std::forward_as_tuple(u, it.value(), *this),
+                         Logger::Domain::SCHEMATIC);
+        }
+    }
+    if (j.count("arcs")) {
+        const json &o = j["arcs"];
+        for (auto it = o.cbegin(); it != o.cend(); ++it) {
+            auto u = UUID(it.key());
+            load_and_log(arcs, ObjectType::ARC, std::forward_as_tuple(u, it.value(), *this), Logger::Domain::SCHEMATIC);
+        }
+    }
+    if (j.count("title_block_values")) {
+        const json &o = j["title_block_values"];
+        for (auto it = o.cbegin(); it != o.cend(); ++it) {
+            title_block_values[it.key()] = it.value();
+        }
+    }
 }
 
-Sheet::Sheet(const UUID &uu) : uuid(uu), name("First sheet"), index(1)
+Sheet::Sheet(const UUID &uu) : uuid(uu), name("First sheet"), index(1), frame(UUID())
 {
 }
 
@@ -185,8 +210,13 @@ void Sheet::expand_symbols(void)
             for (auto &it_pin : pads) {
                 std::sort(it_pin.second.begin(), it_pin.second.end(),
                           [](const auto &a, const auto &b) { return strcmp_natural(a, b) < 0; });
-                for (const auto &pad : it_pin.second) {
-                    it_pin.first->pad += pad + " ";
+                if (it_pin.second.size() <= 3 || schsym.display_all_pads) {
+                    for (const auto &pad : it_pin.second) {
+                        it_pin.first->pad += pad + " ";
+                    }
+                }
+                else {
+                    it_pin.first->pad = it_pin.second.front() + " ... " + it_pin.second.back();
                 }
             }
         }
@@ -248,6 +278,18 @@ void Sheet::simplify_net_lines(bool simplify)
             junctions_with_label.emplace(it.second.junction->uuid);
             it.second.junction->connection_count++;
         }
+        for (const auto &it : lines) {
+            for (auto &it_ft : {it.second.from, it.second.to}) {
+                junctions_with_label.emplace(it_ft->uuid);
+                it_ft->has_via = true;
+            }
+        }
+        for (const auto &it : arcs) {
+            for (auto &it_ft : {it.second.from, it.second.to, it.second.center}) {
+                junctions_with_label.emplace(it_ft->uuid);
+                it_ft->has_via = true;
+            }
+        }
 
         std::map<UUID, std::set<UUID>> junction_connections;
         for (auto &it_line : net_lines) {
@@ -598,6 +640,11 @@ Junction *Sheet::replace_bus_ripper(BusRipper *rip)
     return j;
 }
 
+Junction *Sheet::get_junction(const UUID &uu)
+{
+    return &junctions.at(uu);
+}
+
 /*void Sheet::connect(SchematicSymbol *sym, SymbolPin *pin, PowerSymbol
 *power_sym) {
         auto uu = UUID::random();
@@ -618,8 +665,10 @@ json Sheet::serialize() const
     json j;
     j["name"] = name;
     j["index"] = index;
-    j["frame"] = frame.serialize();
+    if (frame.uuid)
+        j["frame"] = (std::string)frame.uuid;
     j["symbols"] = json::object();
+    j["title_block_values"] = title_block_values;
 
     for (const auto &it : symbols) {
         j["symbols"][(std::string)it.first] = it.second.serialize();
@@ -653,6 +702,14 @@ json Sheet::serialize() const
     for (const auto &it : bus_rippers) {
         j["bus_rippers"][(std::string)it.first] = it.second.serialize();
     }
+    j["lines"] = json::object();
+    for (const auto &it : lines) {
+        j["lines"][(std::string)it.first] = it.second.serialize();
+    }
+    j["arcs"] = json::object();
+    for (const auto &it : arcs) {
+        j["arcs"][(std::string)it.first] = it.second.serialize();
+    }
 
     return j;
 }
