@@ -3,12 +3,15 @@
 #include "nlohmann/json_fwd.hpp"
 #include <string>
 #include <vector>
+#include <functional>
 
 namespace horizon {
 using json = nlohmann::json;
 void save_json_to_file(const std::string &filename, const json &j);
+json load_json_from_file(const std::string &filename);
 int orientation_to_angle(Orientation o);
 std::string get_exe_dir();
+void allow_set_foreground_window(int pid);
 std::string coord_to_string(const Coordf &c, bool delta = false);
 std::string dim_to_string(int64_t x, bool with_sign = true);
 std::string angle_to_string(int angle, bool pos_only = true);
@@ -42,4 +45,11 @@ std::string get_config_dir();
 
 void replace_backslash(std::string &path);
 json json_from_resource(const std::string &rsrc);
+bool compare_files(const std::string &filename_a, const std::string &filename_b);
+void find_files_recursive(const std::string &base_path, std::function<void(const std::string &)> cb,
+                          const std::string &path = "");
+
+Color color_from_json(const json &j);
+json color_to_json(const Color &c);
+
 } // namespace horizon
