@@ -11,10 +11,14 @@ public:
     EntityPreview(class Pool &pool, bool show_goto = true);
 
     void load(const class Entity *entity);
+    void load(const class Part *part);
+    void clear();
 
 private:
+    void load(const class Entity *entity, const class Part *part);
     class Pool &pool;
     const class Entity *entity = nullptr;
+    const class Part *part = nullptr;
     class PreviewCanvas *canvas_symbol = nullptr;
     Gtk::ComboBoxText *combo_gate = nullptr;
     Gtk::ComboBoxText *combo_symbol = nullptr;
