@@ -17,6 +17,7 @@ protected:
     void create_columns() override;
     void add_sort_controller_columns() override;
     UUID uuid_from_row(const Gtk::TreeModel::Row &row) override;
+    PoolItemSource pool_item_source_from_row(const Gtk::TreeModel::Row &row) override;
 
 private:
     class ListColumns : public Gtk::TreeModelColumnRecord {
@@ -29,6 +30,7 @@ private:
             Gtk::TreeModelColumnRecord::add(tags);
             Gtk::TreeModelColumnRecord::add(uuid);
             Gtk::TreeModelColumnRecord::add(path);
+            Gtk::TreeModelColumnRecord::add(source);
         }
         Gtk::TreeModelColumn<Glib::ustring> name;
         Gtk::TreeModelColumn<Glib::ustring> manufacturer;
@@ -36,9 +38,11 @@ private:
         Gtk::TreeModelColumn<Glib::ustring> path;
         Gtk::TreeModelColumn<unsigned int> n_pads;
         Gtk::TreeModelColumn<UUID> uuid;
+        Gtk::TreeModelColumn<PoolItemSource> source;
     };
     ListColumns list_columns;
     Gtk::Entry *name_entry = nullptr;
+    Gtk::Entry *manufacturer_entry = nullptr;
     Gtk::Entry *tags_entry = nullptr;
 };
 } // namespace horizon
